
/* 
 * LiulGuph.m -- Implementation file for the LiulGuph class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to its
 * fitness for any particular use, or even that it has a use.
 *
 */

#import "LiulGuph.h"
#import "drawingFuncs.h"
#import "nxlcau6h.h"
#import "nxardn6h.h"
#import "nxrijn6h.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>
#include <stdio.h>
#include <time.h>

int    cnstRijn=NO;
int    drawBasic=NO;
int    justRijn=NO;
int    shortRijn=NO;

int    dbno=0;		/* node number in de Bruijn diagram */

extern int    drawview;

int    rijnShift=0;
int    rijnGen=1;
int    rijnPerm=0;

extern parneih    amat[];

extern twoparn    smat;
extern twoparn    tmat;
extern twoparn    umat;
extern twoparn    vmat;

extern NXColor colst[KK];

extern double  cntr[][2];	/* de Bruijn diagram coordinates	*/

extern int    dd;		/* radius of de Bruijn ring */
extern char   arule[DS], arr1[AL], arr2[AL], arr3[AL];

void   asctobin();
int    closeto(double x, double y);
void   edijn(int g, int s, int p, int r);
void   geroo(int *root, int *comp, int *link, int cp);
void   howdown(int *sud);
int    level (int n);
int    prnins(int i, twoparn *z);
int    prouts(int i, twoparn *z);
void   prxtrmd(int *min, int *max);
void   prxtrml(int *min, int *max);
int    putlevel(twoparn *zmat, twoparn *xmat, int n);
int    shflev(int g, int s);
void   shortevolve(int m, int n);
void   upthere(int *sud);
void   wrru(char *c, char ru[KK][KK]);


@implementation LiulGuph

/* -------------------------------- R I J N -------------------------------- */

- rijn:sender
{int i;
       if (sender==gen2) rijnGen=0; 
  else if (sender==gen3) rijnGen=1; 
  else if (sender==gen4) rijnGen=2; 
  else if (sender==gen5) rijnGen=3; 
  else if (sender==gen6) rijnGen=4; 
  else                   rijnGen=5; 
  rijnShift=  [sender selectedCol];
  [self rijnReset: self];
  [sender selectCellAt: 0: rijnShift];
  i=shflev(rijnGen,rijnShift);
  [underDiagram setEmptySelectionEnabled: YES];
  [[underDiagram selectedCell] setIntValue: 0];
  [[underDiagram cellAt: 0: i] performClick: self];
  [underDiagram selectCellAt: 0: i];
  [whichConstant setEmptySelectionEnabled: YES];
  [whichConstant selectCellAt: -1: 0];
  rijnPerm=[whichPermutation selectedRow];
  edijn(rijnGen,rijnShift,rijnPerm,![dbBranches state]);
  cnstRijn=NO;
return self;
}

- rijnConst:sender
{
  rijnGen=[sender selectedRow];
  rijnShift=[sender selectedCol];
  [self rijnReset: self];
  [sender selectCellAt: rijnGen: rijnShift];
  [underDiagram setEmptySelectionEnabled: YES];
  [[underDiagram selectedCell] setIntValue: 0];
  [[underDiagram cellAt: 0: rijnGen] performClick: self];
  [underDiagram selectCellAt: 0: rijnGen];
  [whichShift setEmptySelectionEnabled: YES];
  [whichShift selectCellAt: -1: 0];
  rijnPerm=[whichPermutation selectedRow];
  edijn('A'-'a'+rijnGen,rijnShift,rijnPerm,![dbBranches state]);
  cnstRijn=YES;
return self;
}

- rijnReset:sender 
{char c[WNML];
  [gen2 setEmptySelectionEnabled: YES];
  [gen2 selectCellAt: -1: 0];
  [gen3 setEmptySelectionEnabled: YES];
  [gen3 selectCellAt: -1: 0];
  [gen4 setEmptySelectionEnabled: YES];
  [gen4 selectCellAt: -1: 0];
  [gen5 setEmptySelectionEnabled: YES];
  [gen5 selectCellAt: -1: 0];
  [gen6 setEmptySelectionEnabled: YES];
  [gen6 selectCellAt: -1: 0];
  [whichConstant setEmptySelectionEnabled: YES];
  [whichConstant selectCellAt: -1: 0];
  [underDiagram setEmptySelectionEnabled: YES];
  [[underDiagram selectedCell] setIntValue: 0];
  wru(c);
  [ruleNumber setStringValue: c];
return self;
}

- rijnHowBig:sender
{
int    i;
double th, dt;
    switch ([[sender selectedCell] tag]) {
      case 0:  dd=6; break;
      case 1:  dd=36; break;
      case 2:  dd=216; break;
      case 3:  dd=1296; break;
      case 4:  dd=7776; break;
      default: dd=25; break;}
  dt=6.28318/((double)(dd));
  for (i=0, th=0.5*dt; i<dd; i++, th+=dt) {
    cntr[i][0]=200.0-180.0*sin(th);
    cntr[i][1]=200.0-180.0*cos(th);
    }

return self;
}

- rijnRanSam:sender
{
int  i;
  i='z';
  if (strcmp([[sender selectedCell] title],"random sample")==0) i='r';
  if (strcmp([[sender selectedCell] title],"selected node")==0) i='n';
  if ([dbBranches state]) return self;
  if (cnstRijn) edshc(rijnGen); edshf(rijnGen,rijnShift,i);
  [evWindow makeKeyAndOrderFront: sender];
  [evButton performClick: sender];
return self;
}

/* load a panel for selecting the type of de Bruijn diagram */

- loadDiagramType:sender 
{char *ch;
  ch=[[sender selectedCell] title];
  if (strcmp(ch,"shift")==0)     [self loadSelectionPanel: shiftBox];
  if (strcmp(ch,"constant")==0)  [self loadSelectionPanel: constBox];
return self;
}

- loadSelectionPanel:rijnNewBox 
{NXRect bigRect, lilRect;
 id cv;
 cv=[rijnNewBox contentView];
  [rijnBigBox getFrame: &bigRect];
  [rijnNewBox getFrame: &lilRect];
  [rijnLilBox setContentView: cv];
  lilRect.origin.x=
	bigRect.origin.x+(bigRect.size.width-lilRect.size.width)/2.0;
  lilRect.origin.y=
	bigRect.origin.y+(bigRect.size.height-lilRect.size.height)/2.0;
  [rijnLilBox setFrame: &lilRect];
  [rijnBigBox display];
  [rijnLilBox display];
return self;
}

- basicRijn:sender 
{char c[WNML], d[37+WNML];
  drawview=0;
  drawBasic=YES;
		[self rijnReset:self];
  		[[underDiagram cellAt: 0: 0] performClick: self];
  wru(c);
  sprintf(d,"lcau6h rule  %s  de Bruijn diagram",c);
  [sparePanel makeKeyAndOrderFront:self];
  [sparePanel setTitle: d];
  [spareView display];
  drawBasic=NO;
return self;
}

- Zot:sender
{
    drawview=0;
    shortRijn=YES;
    [self display];
    shortRijn=NO;
return self;
}

- shewRijn:sender
{
    drawview=0;
    justRijn=YES;
    [self display];
    justRijn=NO;
return self;
}

- printBasic:sender 
{char c[WNML], d[37+WNML];
  drawview=0;
  drawBasic=YES;
		[self rijnReset:self];
  		[[underDiagram cellAt: 0: 0] performClick: self];
  wru(c);
  sprintf(d,"lcau6h rule  %s  de Bruijn diagram",c);
  [sparePanel makeKeyAndOrderFront:self];
  [sparePanel setTitle: d];
  [spareView display];
  [sparePanel smartPrintPSCode:self];
  drawBasic=NO;
return self;
}

- printGeneric:sender 
{
  drawview=0;
  justRijn=YES;
  [sparePanel makeKeyAndOrderFront:self];
  [sparePanel setTitle: "generic lcau6h de Bruijn diagram"];
  [spareView display];
  [sparePanel smartPrintPSCode:self];
  justRijn=NO;
return self;
}

- printRijn:sender
{
  drawview=0;
  shortRijn=YES;
  [self printPSCode:self];
  shortRijn=NO;
return self;
}

/* - - - - - - - - - - - - -    L I U L G U P H   - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new LiulGuph object.  First, an initFrame: message is sent 
 * to super to initialize LiulGuph as a View.  Next, the LiulGuph sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];
/*	[self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	crossCursor = 
		[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}


- setDelegate:anObject
/*
 * Sets the LiulGuph's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}


- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the LiulGuph's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although LiulGuph only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[30+WNML], timday[30];
	int		 i, j, k;
        double           p, q, s, dp;
  struct tm  *timeptr;
  time_t  secsnow;
	NXPoint		 *aPoint;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* 
	 * If we're printing, we need to load drawing procedures to the 
	 * printing context.  This simple test is OK for a View that prints a 
	 * single page.  If there were multiple pages, the procedures would be 
	 * loaded multiple times.  Although this is inefficient, it causes no 
	 * error.  Views that print multiple pages should load procedures such 
	 * as these in an endPrologue method (see the View spec sheet). 
	 */
	if (NXDrawingStatus != NX_DRAWING) 
		loadPSProcedures();

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

// drawAxes(bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height);

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(timday,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [timeDate setStringValue: timday];

/* for the de Bruijn  diagram --- - - - - - - - - - - - - - - - - - - - - */

        if (drawBasic==YES) {
		PSsetgray(NX_BLACK); 
   [[Font newFont: "Times-Bold" size: 36.0 matrix:NX_FLIPPEDMATRIX] set];
		for (i=0; i<KK; i++) sijn(i,i); 
		PSstroke();
		}

        if (justRijn==YES) {
		PSsetgray(NX_BLACK); 
		ijn(dd,KK,1); 
		PSstroke();
		}

        if (shortRijn==YES) {
   [[Font newFont: "Courier" size: 20.0 matrix:NX_FLIPPEDMATRIX] set];
   wru(c);
   [ruleNumber setStringValue: c];
   PSmoveto(290.0,14.0);
   PSshow(c);
   PSmoveto(12.0,14.0);
   j=rijnShift-3;
   if (rijnGen%2==0 && j>=0) j++;
   [[Font newFont: "Courier" size: 12.0 matrix:NX_FLIPPEDMATRIX] set];
   if (cnstRijn) sprintf(c,"evolving to %1d in %1d gen",rijnShift,rijnGen+1); 
   else {if (rijnPerm==0) sprintf(c,"shift %1d in %1d gen",j,rijnGen+1);
   else sprintf(c,"shift %1d in %1d gen perm %d",j,rijnGen+1,rijnPerm);}
   PSshow(c);
   [[Font newFont: "Courier" size: 10.0 matrix:NX_FLIPPEDMATRIX] set];
   PSsetgray(NX_BLACK); 
   if (cnstRijn) edcst(rijnGen); else edshg(rijnGen,rijnShift);
   PSstroke();

		}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the LiulGuph is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo:width :height];
	if (drawview==4||drawview==7) {[self setDrawOrigin:0 :0]; return self;}
	if (drawview!=4) [self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the LiulGuph.  The LiulGuph changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the LiulGuph updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the LiulGuph, the LiulGuph 
 * registers the point and then sends a message to its delegate notifying it of 
 * the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect, cellRect;
   	char	buffer[100];

         double  x, y, ex, wy, eks, wye;
	 int     i, j, k;

	[crossCursor set];
   	[self getBounds:&plotRect];
 	NXSetRect(&cellRect, plotRect.origin.x, plotRect.origin.y, 100.0, 20.0);

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];

/* for the de Bruijn  diagram --- - - - - - - - - - - - - - - - - - - - - */

 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				dbno=nearest(ex,wy);
				[nodeNumber setIntValue: dbno];

       if (theEvent->data.mouse.click==3) {} else 
       if (theEvent->data.mouse.click==2) {} else {
	 [self lockFocus];
         PSsetinstance(YES);
	do {
 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
                ex=aPoint.x;
                wy=aPoint.y;
		sprintf(buffer, "(%d, %d)", (int)aPoint.x, (int)aPoint.y); 
				PSsetgray(NX_BLACK);
				PSmoveto(ex+12.0,wy);
				PSarc(ex, wy, 12.0, 0.0, 360.0);
				PSstroke();
				cntr[dbno][0]=ex;
				cntr[dbno][1]=wy;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {looping = NO;}	
   	} while (looping && 
	  (theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];


   shortRijn=YES;
//   [window display];
   [self display];
   shortRijn=NO;

	}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

- (const char *)inspectorName
{
    return "LiulGuphInspector";
}

@end

/* end LiulGuph.m */
