
/* 
 * OverAll.m -- Implementation file for the OverAll class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "OverAll.h"
#import "nxlcau6h.h"
#import "rec.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>
#import <time.h>

int  co, dn;			/* connectivity, dimension */

char dfclr[KK][12] = {"1.0,0.0,0.0",
		      "0.0,1.0,0.0",
		      "0.0,0.0,1.0",
		      "0.5,0.0,0.5",
		      "0.5,0.5,0.0",
		      "0.2,0.4,0.4"
		     };

NXColor colst[KK];

NXDefaultsVector defaults={
  {"well0","1.0,0.0,0.0"},
  {"well1","0.0,1.0,0.0"},
  {"well2","0.0,0.0,1.0"},
  {"well3","0.5,0.0,0.5"},
  {"well4","0.5,0.5,0.0"},
  {"well5","0.2,0.4,0.4"},
  {NULL,NULL}
  };

 char colorwells[KK][6]={"well0",
			 "well1",
			 "well2",
			 "well3",
			 "well4",
			 "well5"};
  
extern struct fptbl dtbl[];
extern Inst xprg[XLEN];
extern char idef[][30];


@implementation OverAll

/* ---------------  A P P   I N I T I A L I Z A T I O N   ----------------- */

- appWillInit:sender {printf("appWillInit\n"); return self;}

- appDidInit:sender 
{
char c[WNML];           /* nombre de la regla trece digitos + control de fin */
struct tm  *timeptr;    /* apuntador de la hora */
time_t  secsnow; 
char *df;
double red,green,blue;       
int i;

  printf("appDidInit = nxlcau6h\n");
  co=1;		/* normal connectivity */
  dn=1;		/* one-dimensional     */
  rule6h();       /* printf("rule6h\n"); toma aleatoriamente una regla de 6h */
  asctobin();    /* pasa la regla dada de ascii a binario */      
  ttzero();      /* inicializa la regla totalistica a cero */
  gfzero();
  [rulePanel setAllRules: self];
  [dbguph rijnReset: self];

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(c,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: c];
  NXRegisterDefaults("NXLCAU6HDefaults",defaults);
      NXUpdateDefaults();
      for (i=0; i<KK; i++){
       df=NXGetDefaultValue("NXLCAU6HDefaults",colorwells[i]);
       sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); 
       colst[i]=NXConvertRGBToColor(red,green,blue);
       printf("%s: %s\n",colorwells[i],df);
       }
      [[NXColorPanel sharedInstance:YES]setContinuous: YES];

  /* initialize r_vst with selected standard procedures */ 
       rec_c(idef[0],xprg[0],128,dtbl);
       r_vst['a']=xprg[0];

  [browser getTitleFromPreviousColumn: YES];
  [browser setDoubleAction: @selector(copyAndRunREC:)];
  [browser setPath: "/demonstrations"];

   printf("end appDidInit\n");
return self;
}

- appWillTerminate:sender
{return self;}

- awakeFromNib 
{
printf("awaking from nib\n");
return self;
}

- setUpWell: well color: (NXColor)aColor tag: (int)aTag 
{
  [well setColor: aColor];
  [well setTag: aTag];
  [well setContinuous: YES];
return self;
}

/* - - - - - - - -  R E C   L I N E   C O N F I G U R A T I O N  - - - - - */
 
- useRECLine:sender
{
  [RECPanel makeKeyAndOrderFront: sender];
  [RECValue setStringValue: "*"];
 return self;
}

- runRECLine:sender;
{
  [RECValue setStringValue: " "];
  if (rec([sender stringValue])) {
     [RECValue setStringValue: "T"];
     [rulePanel setAllRules: self];
     }
  else
    [RECValue setStringValue: "F"];
  return self;
}

/* << S E T - D I M E N S I O N - A N D - L O A D - A C C O R D I N G L Y >> */

- dimenChoice:sender {
   if (dimenPrefs==nil) {
      if (![NXApp loadNibSection: "overall.nib" owner: self withNames: NO]) 
	{return nil;}
      [dimenMenu setEmptySelectionEnabled: YES];
      [dimenMenu selectCellAt: -1: 0];
    }
  [dimenPrefs makeKeyAndOrderFront: sender];
  return self;
}

- setDimension:sender {dn = [sender selectedRow]+1; return self;}

- dbWindow:sender 
{
printf("ruinWindow is about to load %d\n",dn);
   switch (dn) {
     case 2:  [ruinView showRuin: sender];
//	      [db2Window performClose: sender];
//	      [db3Window performClose: sender];
	      break;
     default: break;}
printf("ruinWindow is loaded\n");
  return self;
}



@end
