
/* 
 * PairView.m -- Implementation file for the PairView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to its
 * fitness for any particular use, nor even that it has a use.
 *
 */

#import "PairView.h"
#import "drawingFuncs.h"
#import "nxlcau6h.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>
#include <stdio.h>
#include <time.h>


extern int    drawview;

extern parneih    amat[];

extern twoparn    smat;
extern twoparn    tmat;
extern twoparn    umat;
extern twoparn    vmat;

extern NXColor colst[KK];

int    prno=0;			/* node number in pair diagram		*/
int    prlink[KK][K2];		/* links in pair diagram	 	*/
int    prdown[PDN];		/* depth of pair diagram nodes		*/
int    prhigh[PDN];		/* height of pair diagram nodes		*/
int    prnd;			/* minimum pair separation		*/
int    prxd;			/* maximum pair separation		*/
int    prnl;			/* minimum loop length			*/
int    prxl;			/* maximum loop length			*/
int    ergset[PDN];		/* members of the ergodic set		*/
double pcntr[PDN][2];		/* pair diagram coordinates		*/

extern char   arule[DS], arr1[AL], arr2[AL], arr3[AL];

int    anti(int n);
void   asctobin();
int    closeto(double x, double y);
void   edPair(int l, int pc, int pa, int pl, int pv, int op);
void   finderset(int *e, twoparn *t);
void   pairflip();
void   howdown(int *sud);
int    level (int n);
int    pmax(int i, int j);
int    pmin(int i, int j);
int    prnins(int i, twoparn *z);
int    prouts(int i, twoparn *z);
void   prxtrmd(int *min, int *max);
void   prxtrml(int *min, int *max);
int    putlevel(twoparn *zmat, twoparn *xmat, int n);
int    shflev(int g, int s);
void   tensq(twoparn *tmat, parneih amat[]);
void   suanc();
void   upthere(int *sud);

@implementation PairView


/* -------------------------------- P A I R -------------------------------- */

/* put up the principal window for the PairView. Since pair.nib is a 	   */
/* secondary .nib file, it is not loaded until there is a demand, which    */
/* would usually arise from the "Pair" option in the main menu.	   */

- showPair:sender {
  if (pairWindow==nil) {
    if (![NXApp loadNibSection: "pair.nib" owner:self withNames: NO]) 
	{return nil;}
    }
  [pairWindow makeKeyAndOrderFront: nil];
  return self;
  }

- drawPair:sender
{
int    i, j, k;
printf("drawPair\n");
asctobin();
for (i=0; i<KK; i++) zmat(&amat[i]);
for (i=0; i<KK; i++) djmat(&amat[i],i);
zzmat(&tmat); tensq(&tmat,amat);
printf("tensq\n");
smat=umat=tmat; 
  i=2; do {
    mmmat(&vmat,&umat,&tmat);
    k=putlevel(&smat,&vmat,i);
    umat=vmat;
    i++;
printf("running level %3d\n",k);
    } while (k!=0);
  for (i=0; i<K2; i++) for (j=0; j<K2; j++) 
	[[prTriv2 cellAt: i: j] setIntValue: smat.two[i][j]];
  prxtrmd(&prnd,&prxd);
  [prTriv1 setIntValue: prnd at: 2];
  [prTriv1 setIntValue: prxd at: 3];
  prxtrml(&prnl,&prxl);
  [prTriv1 setIntValue: prnl at: 0];
  [prTriv1 setIntValue: prxl at: 1];
  umat=tmat;
  for (i=0; i<PDN; i++) ergset[i]=1;
  for (i=0; i<5; i++) finderset(ergset,&umat);

  for (i=0; i<KK; i++) for (j=0; j<KK; j++) {
    pcntr[i+KK*j][0]=(double)(35+60*i);
    pcntr[i+KK*j][1]=(double)(35+60*j);
    }
  [self redrawPair: self];
return self;
}

- redrawPair:sender
{
  [nodeNumber setIntValue: prno];
  if ([ordPair state]) pairflip();
  drawview=0;
  [self display];
return self;
}

- printPair:sender
{
  drawview=0;
  [wholePanel smartPrintPSCode:self];
return self;
}

int putlevel(zmat,xmat,n) twoparn *zmat, *xmat; int n; {int i, k;
k=0;
for (i=0; i<K4; i++) 
  if (zmat->one[i]==0 && xmat->one[i]!=0) {zmat->one[i]=n; k=1;}
return k;
}

void edPair(l,pc,pa,pl,pv,op) int l, pc, pa, pl, pv, op; {
int    i, j, m;
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, t;

  m=0;
  r=10.3;
  for (i=0; i<PDN; i++) {
     			if (pv && ergset[i]==0) continue; 
    for (j=0; j<PDN; j++) {
     			if (pv && ergset[j]==0) continue; 
	if (tmat.two[i][j]==0) continue;
	m++;
	ex=pcntr[i][0];
	wy=pcntr[i][1];
	PSmoveto(ex,wy);
	x=pcntr[j][0];
	y=pcntr[j][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	eks=x-r*cos(t);
	wye=y-r*sin(t);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSmoveto(x+10.3,y);
	PSarc(x, y, 10.3, 0.0, 360.0);
	PSmoveto(ex+10.3,wy);
	PSarc(ex, wy, 10.3, 0.0, 360.0);
	}

   if (pl==YES) {
   ex=pcntr[i][0];
   wy=pcntr[i][1];
   PSmoveto(ex-10.0,wy+5.0);
	if (op) sprintf(s,"(%1d,%1d)",pmin(i%KK,i/KK),pmax(i%KK,i/KK));
	   else sprintf(s,"(%1d,%1d)",i%KK,i/KK);
   PSshow(s);
   }
  }
//	printf("total links = %5d\n",m);
}

/* find index of nearest center to x, y (cycles) */
int nearto(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<PDN; i++) {
  ex=x-pcntr[i][0];
  wy=y-pcntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* find node number of the reflected pair */
int anti(n) int n; {return (n/KK)+KK*(n%KK);}

/* min, max of two integers */ 
int pmin(i,j) int i, j; {return i<j?i:j;}
int pmax(i,j) int i, j; {return i>j?i:j;}

/* flip the location of the disordered members of symmetric pairs */ 
void pairflip() {int i, j;
  for (i=1; i<KK; i++) for (j=0; j<i; j++) {
    pcntr[i+KK*j][0]=pcntr[j+KK*i][0];
    pcntr[i+KK*j][1]=pcntr[j+KK*i][1];
    }
}

/* calculate number of incoming links for pair diagram nodes */
int prnins(i,z) int i; twoparn *z; {int j, k;
for (k=0, j=0; j<PDN; j++) if (z->two[j][i]!=0) k++;
return k;
}

/* calculate number of incoming links for pair diagram nodes */
int prouts(i,z) int i; twoparn *z; {int j, k;
for (k=0, j=0; j<PDN; j++) if (z->two[i][j]!=0) k++;
return k;
}

/* if there are no ins to i, make link(i,j)=0 for all j */
void finderset(e,t) int *e; twoparn *t; {int i, j; twoparn z;
z=*t;
for (i=0; i<PDN; i++) if (prnins(i,&z)==0) {
  e[i]=0;
  for (j=0; j<PDN; j++) z.two[i][j]=0;
  }
for (i=0; i<PDN; i++) if (prouts(i,&z)==0) {
  e[i]=0;
  for (j=0; j<PDN; j++) z.two[j][i]=0;
  }
*t=z;
}

/* find extremal distances in the pair distance matrix */
void prxtrmd(min, max) int *min, *max; {int i, n, x, s;
x=0; n=K4;
for (i=0; i<K4; i++) {s=smat.one[i]; if (s<n) n=s; if (s>x) x=s;}
*min=n; *max=x;
}

/* find extremal loops in the pair distance matrix */
void prxtrml(min, max) int *min, *max; {int i, n, x, s;
x=0; n=K4;
for (i=0; i<K2; i++) {s=smat.two[i][i]; if (s<n) n=s; if (s>x) x=s;}
*min=n; *max=x;
}

/* - - - - - - - - - - - - -    P A I R V I E W   - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new PairView object.  First, an initFrame: message is sent 
 * to super to initialize PairView as a View.  Next, the PairView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];
/*	[self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	crossCursor = [NXCursor newFromImage:
		[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];
	loadPSProcedures();

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}

- setDelegate:anObject
/*
 * Sets the PairView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the PairView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although PairView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[30], timday[30];
	unsigned int	 i, j, k;
        double           p, q, s, dp;
  struct tm  *timeptr;
  time_t  secsnow;
	NXPoint		 *aPoint;

int     i0, i1, ss;
	
	if (rects == NULL) return self;

	/* 
	 * If we're printing, we need to load drawing procedures to the 
	 * printing context.  This simple test is OK for a View that prints a 
	 * single page.  If there were multiple pages, the procedures would be 
	 * loaded multiple times.  Although this is inefficient, it causes no 
	 * error.  Views that print multiple pages should load procedures such 
	 * as these in an endPrologue method (see the View spec sheet). 
	 */
	if (NXDrawingStatus != NX_DRAWING) 
		loadPSProcedures();

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(timday,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);

/* for the pair diagram -- - - - - - - - - - - - - - - - - - - - - - - - - - */

  	[[Font newFont: "Courier" size: 10.0 matrix:NX_FLIPPEDMATRIX] set];
   wru(c);
   [ruleNumber setStringValue: c];

   [timeDate setStringValue: timday];

		PSsetlinewidth(0.0);
		PSsetgray(NX_BLACK); 
		edPair(0,YES,YES,YES,[treeShow state],[ordPair state]);
		PSstroke();
   [timeDate setStringValue: timday];
		PSstroke();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the PairView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo:width :height];
	if (drawview==4||drawview==7) {[self setDrawOrigin:0 :0]; return self;}
	if (drawview!=4) [self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the PairView.  The PairView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the PairView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the PairView, the PairView 
 * registers the point and then sends a message to its delegate notifying it of 
 * the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;

         double  x, y, ex, wy, eks, wye;
	 int     i, j, k;

	[crossCursor set];

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];

/* for the pair diagram  - - - - - - - - - - - - - - - - - - - - - - - - - - */

 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				prno=nearto(ex,wy);
				[nodeNumber setIntValue: prno];

       if (theEvent->data.mouse.click==3) {} else 
       if (theEvent->data.mouse.click==2) {} else {
	[self lockFocus];
         PSsetinstance(YES);
	do {
 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
				PSmoveto(ex+12.0,wy);
				PSarc(ex, wy, 12.0, 0.0, 360.0);
				PSstroke();
				pcntr[prno][0]=ex;
				pcntr[prno][1]=wy;
				if ([ordPair state]) {
					k=anti(prno);
					pcntr[k][0]=ex;
					pcntr[k][1]=wy;
					}
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	    	/* on mouse-up, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent=[NXApp getNextEvent:
		NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

   [self display];
	}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

- read:(NXTypedStream *)stream
/*
 * Unarchives the PairView.  Initializes four of the PairView's instance 
 * variables to the values stored in the stream. 
 */
{
	[super read:stream];
	delegate = NXReadObject(stream);
	return self;
}

- write:(NXTypedStream *)stream
/*
 * Archives the PairView by writing its important instance variables to the 
 * stream. 
 */
{
	[super write:stream];
	NXWriteObjectReference(stream, delegate);
	return self;
}

- awake
/*
 * Finishes initializing a PairView that has just been unarchived (see the 
 * read: method).
 */
{
	loadPSProcedures();
	return [super awake];
}


- (const char *)inspectorName
{
    return "PairViewInspector";
}

@end

/* end PairView.m */
