
/* lcau51.c				*/
/* (5,1) Linear Cellular Automaton	*/

/* Reference:				*/
/*					*/
/*    Kenneth E. Perry			*/
/*    Abstract Mathematical Art		*/
/*    BYTE				*/
/*    December, 1986			*/
/*    pages 181-192			*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Harold V. McIntosh		*/
/*    Gerardo Cisneros S.		*/

/* G. Cisneros, 4.3.87						*/
/* 28 April 1987 - version for XVI Feria de Puebla [HVM]	*/
/* 14 May 1987 - modified for (3,1) and general rule [HVM]	*/
/* 19 May 1987 - modified for (2,2) [HVM]			*/
/* 20 May 1987 - modified for (2,3) [HVM]			*/
/* 8 June 1987 - general rule for (4,1) [HVM]		 	*/
/* 12 June 1987 - cartesian product of (2,1) rules [HVM]	*/
/* 12 June 1987 - (2,1) rule with memory  [HVM]			*/
/* 14 June 1987 - individual cycles of evolution [HVM]		*/
/* 17 June 1987 - p-adic representation in plane [HVM]		*/
/* 22 June 1987 - 2 speed gliders via 16 transitions [HVM]	*/
/* 26 June 1987 - push, pop the rule [HVM]			*/
/* 26 June 1987 - conserve position of rule cursor [HVM]	*/
/* 27 June 1987 - incremental rule construction [HVM]		*/
/* 29 June 1987 - conserve position of cell pointer [HVM]	*/
/* 30 June 1987 - mark & unmark transitions, xchg x&X [HVM]	*/
/* 25 July 1987 - display and edit de Bruijn diagrams [HVM]	*/
/* 27 July 1987 - graph of transition probabilities [HVM]	*/
/* 4 September 1987 - PROB41.C for option 't' [HVM]		*/
/* 21 October 1987 - program disks disappeared			*/
/* 24 December 1987 - program reconstructed from listings	*/
/* 20 February 1988 - RIJN41.C for option 'd' [HVM]		*/
/* 4 April 1989 - adapted for (5,1) automata [HVM]		*/
/* 22 September 1989 - edline adjusted for design [HVM]		*/
/* 18 October 1989 - rule search for prescribed evolution [HVM]	*/

# include <bdos.h>
# include "lcau51.h"

# define LROW		 9	/* top row of line panel */
# define DROWS		20	/* rows in demo menu	 */
# define DCOLS		32	/* second col in demo m. */

# define NX          	 6	/* number of sample rules    */

char ixrule[NX][TS]= {

	"0001321410401",	/* */
	"0001321410401",	/* */
	"0001322410400",	/* */
	"0003203102422",	/* */
	"0102340432100",	/* */
	"0021032403234"		/* */

	};

char   trule[TS];
char   tabl[20][SL];				/* workspace to construct table */
char   ascrule[KK][KK][KK];			/* ASCII transition values */
char   auxrule[KK][KK][KK];			/* auxiliary transition table */
char   rulstk[10][KK][KK][KK];			/* pushdown for rules */
char   prule1[8], prule2[8];			/* product rule specification */
int    binrule[KK][KK][KK];
int    arr1[AL], arr2[AL];			/* line of cells */
int    ru, ru0, ru1, ru2;			/* rule cursor */
int    di, dj;					/* demo pointer */
int    li, lj;					/* cell pointer */
int    rulptr;					/* rule pd ptr */
int    sc;					/* screen colors */
int    ix0, iy0;				/* origin for pen moves */
double wmul[KK], wmvl[KK];			/* left mass point */
double wmur[KK], wmvr[KK];			/* right mass point */


main() {
int  i, j, i0, i1, i2, i3, i4, i5;
int  more = 'r';
char a, b, c;

ru=6; ru0=0; ru1=0; ru2=0;
li=SL/2; lj=0;
sc=0;
rulptr=0;

for (i=0; i<TS; i++) trule[i]='0';

    videopalette(WHCYMAG);				/* white/cyan/magenta */

    tuto();
    while (!kbdst()) rand();				/* wait for keypress */
    kbdin();						/* ignore it */
    videomode(T80X25);
    videoscroll(3,0,5,71,0,3);				/* menu on blue background */
    videoscroll(19,0,24,71,0,3);
    i=(rand()%NX);
    di=i%DROWS;
    dj=(i/DROWS==1?DCOLS:1);
    xtoasc(i);
    ranlin();						/* random initial array */
    auxblnk();						/* uncomitted transitions */

    while (more!='n') {					/* execute multiple runs */
    rmenu();
    lmenu();
    while (0<1) {					/* set up one run */
    c=kbdin();
    if (c=='g') break;					/* go draw graph */
    if (c=='q') more='n';				/* quit for good */
    if (more=='n') break;
    switch (c) {
        case '!': sc=7; break;
	case '*': sc=2; break;				/* white on red */
	case '+': sc=1; break;				/* white screen */
	case '-': sc=0; break;				/* blue screen */
	case '@':					/* numbered tot rule */
	    nutoto(numin(0));
	    totoasc();
	    rmenu();
	    videocursor(0,4,0);
	    break;
	case '$':					/* dozen totalistics */
	    j=numin(0);
	    for (i=0; i<12; i++) {
	      nutoto(j+i);
	      totoasc();
	      ranlin();
	      evolve();
	      }
	    videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
	case '.':					/* one cycle of evolution */
	    asctobin();
	    onegen(AL);
	    clmenu();
	    pprob();
	    break;
	case ',':					/* one cycle of evolution */
	    videomode(COLGRAF);
	    pgrid();
	    for (i=0; i<200; i++) {pprob(); onegen(AL);}
	    videodot(190,195,3);
	    kbdin();
	    videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
	case 'T':					/* totalistic rule */
	    xblnk();
	    tmenu();
	    edtrule();
	    totoasc();
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	      ascrule[i0][i1][i2]=trule[i0+i1+i2];
	    videocursor(0,4,0);
	    rmenu();
	    xmenu(totonu(0));
	    break;
	case 't':					/* triangular probability graph */
	    edtri();
	    rmenu();
	    lmenu();
	    break;
	case 'p':					/* product of two (2,1) rules */
	    nutowo1(numin(0));
	    nutowo2(numin(0));
	    for (i0=0; i0<2; i0++)
	    for (i1=0; i1<2; i1++)
	    for (i2=0; i2<2; i2++)
	    for (i3=0; i3<2; i3++)
	    for (i4=0; i4<2; i4++)
	    for (i5=0; i5<2; i5++)
	      ascrule[2*i0+i1][2*i2+i3][2*i4+i5]='0'+2*(prule1[4*i0+2*i2+i4]-'0')+(prule2[4*i1+2*i3+i5]-'0');
	    xblnk();
	    rmenu();
	    break;
	case 'm':					/* (2,1) rules with memory */
	    nutowo1(numin(0));
	    for (i0=0; i0<2; i0++)
	    for (i1=0; i1<2; i1++)
	    for (i2=0; i2<2; i2++)
	    for (i3=0; i3<2; i3++)
	    for (i4=0; i4<2; i4++)
	    for (i5=0; i5<2; i5++)
	      ascrule[2*i0+i1][2*i2+i3][2*i4+i5]='0'+2*(prule1[4*i0+2*i2+i4]-'0')+i2;
	    xblnk();
	    rmenu();
	    break;
        case 'r':					/* edit rule */	
	    xblnk();
	    edrule();
	    videocursor(0,4,0);
	    rmenu();
	    break;
        case 'l':					/* edit cell string */
	    xblnk();
	    edline(8,40);
	    videocursor(0,3,0);
	    lmenu();
	    break;
        case '#':					/* read stored rule */
	    xtoasc(lim(1,dmenu(),NX)-1);
	    rmenu();
	    lmenu();
	    break;
	case 'D':					/* run through samples */
	    for (i=0; i<NX; i++) {
	      xmenu(i);
	      xtoasc(i);
	      ranlin();
	      evolve();
	      }
	    videomode(T80X25);
	    rmenu();
	    break;
	case 'd':					/* de Bruijn diagram */
	    edijn();
	    rmenu();
	    lmenu();
	    break;
        case 'u':					/* sparse init arry */
	    xblnk();
	    for (i=0; i<AL; i++) arr1[i]=0;
	    arr1[AL/4]=1;
            arr1[AL/2]=1;
            arr1[(3*AL)/4]=1;
            arr1[(3*AL)/4+2]=1;
	    lmenu();
            break;
	case 'X':					/* random rule */
	    xblnk();
	    i=rand();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	      if (i == 0) i=rand();
	      ascrule[i0][i1][i2]='0'+i%KK;
	      i/=KK;
	    }}}
	    rmenu();
	    break;
	case 'x':					/* random rule */
	    xblnk();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	      if (i == 0) i=rand();
	      if (auxrule[i0][i1][i2]==' ') {
		ascrule[i0][i1][i2]='0'+i%KK;
		i/=KK;
		}
	      }}}
	    rmenu();
	    break;
	case 'y':					/* random line */
	    xblnk();
	    ranlin();
            lmenu();
	    break;
	case 'z':
	    for (i=0; i<AL; i++) arr1[i]=0;
	    lmenu();
	    break;
	case 'Y':					/* symmetrize rule */
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    ascrule[i2][i1][i0]=ascrule[i0][i1][i2];      
	    }}}
	    rmenu();
	    break;
	case 'B':					/* begin barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[0][a-'0'][b-'0']=a;
	    ascrule[1][a-'0'][b-'0']=a;
	    ascrule[2][a-'0'][b-'0']=a;
	    ascrule[3][a-'0'][b-'0']=a;
	    rmenu();
	    break;
	case 'E':					/* end barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[a-'0'][b-'0'][0]=b;
	    ascrule[a-'0'][b-'0'][1]=b;
	    ascrule[a-'0'][b-'0'][2]=b;
	    ascrule[a-'0'][b-'0'][3]=b;
	    rmenu();
	    break;
	case 'L':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][b-'0'][c-'0']=c;
	    rmenu();
	    break;
	case 'R':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][b-'0'][c-'0']=a;
	    rmenu();
	    break;
	case 'S':					/* still life link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][b-'0'][c-'0']=b;
	    rmenu();
	    break;
	case 'U':					/* push rule */
	    if (rulptr<10) rulptr++;
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    rulstk[rulptr][i0][i1][i2]=ascrule[i0][i1][i2];
	    }}}
	    xmenu(rulptr);
	    break;
	case 'V':					/* pop rule */
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    ascrule[i0][i1][i2]=rulstk[rulptr][i0][i1][i2];
	    }}}
	    if (rulptr>0) rulptr--;
	    rmenu();
	    break;
	case 'G':					/* fetch rule */
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    ascrule[i0][i1][i2]=rulstk[rulptr][i0][i1][i2];
	    }}}
	    rmenu();
	    break;
	case '1':
	    ascrule[0][0][0]='0';
	    ascrule[0][0][1]='0';
	    ascrule[0][0][2]='0';
	    ascrule[0][0][3]='1';
	    ascrule[0][1][0]='1';
	    ascrule[0][1][1]='2';
	    ascrule[0][1][3]='1';
	    ascrule[0][2][0]='3';
	    ascrule[0][3][1]='1';
	    ascrule[1][0][0]='1';
	    ascrule[1][1][1]='0';
	    rmenu();
	    break;
	case '2':
	    ascrule[0][0][0]='0';
	    ascrule[0][0][1]='2';
	    ascrule[0][0][2]='0';
	    ascrule[0][0][3]='3';
	    ascrule[0][1][0]='0';
	    ascrule[0][1][3]='2';
	    ascrule[0][2][0]='1';
	    ascrule[0][2][2]='2';
	    ascrule[0][3][0]='0';
	    ascrule[1][0][0]='0';
	    ascrule[1][0][3]='3';
	    ascrule[1][3][0]='0';
	    ascrule[2][0][0]='0';
	    ascrule[2][0][3]='3';
	    ascrule[2][2][0]='2';
	    ascrule[3][0][0]='0';
	    rmenu();
	    break;
	case '=':
	    for (i=1; i<8;  i++) {
	    for (j=0; j<SL; j++) arr1[SL*i+j]=arr1[j];}
	    lmenu();
	    break;
	case ':':
	    for (i=1; i<16;  i++) {
	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];}
	    lmenu();
	    break;
	case ';':
	    for (i=1; i<32;  i++) {
	    for (j=0; j<10; j++) arr1[10*i+j]=arr1[j];}
	    lmenu();
	    break;
        default: break;
        }
    }
    if (more=='n') break;
    do {
    evolve();
    videocursor(0,0,0);
    scrstr("?");
    videocursor(0,0,34);
    scrstr("y/n/cr");
    more=kbdin();
    } while (more=='\015');
    videomode(T80X25);					/* reset the screen */
    if (more=='n') break;
    }
  videomode(T80X25);}    

/* list of demonstration rules */
int dmenu() {int ii, jj; char c;
videomode(T80X25);
videocursor(0,0,0); scrstr(" 1 - ");
videocursor(0,1,0); scrstr(" 2 - ");
videocursor(0,2,0); scrstr(" 3 - ");
videocursor(0,3,0); scrstr(" 4 - ");
videocursor(0,4,0); scrstr(" 5");
videocursor(0,5,0); scrstr(" 6");
videocursor(0,6,0); scrstr(" 7");
videocursor(0,7,0); scrstr(" 8");
videocursor(0,8,0); scrstr(" 9");
videocursor(0,9,0); scrstr("10");
videocursor(0,10,0); scrstr("11");
videocursor(0,11,0); scrstr("12");
videocursor(0,12,0); scrstr("13");
videocursor(0,13,0); scrstr("14");
videocursor(0,14,0); scrstr("15");
videocursor(0,15,0); scrstr("16");
videocursor(0,16,0); scrstr("17");
videocursor(0,17,0); scrstr("18");
videocursor(0,18,0); scrstr("19");
videocursor(0,19,0); scrstr("20");
videocursor(0,0,DCOLS); scrstr("21 ");
videocursor(0,1,DCOLS); scrstr("22 ");
videocursor(0,2,DCOLS); scrstr("23 ");
videocursor(0,3,DCOLS); scrstr("24 ");
videocursor(0,4,DCOLS); scrstr("25 ");
videocursor(0,5,DCOLS); scrstr("26 ");
videocursor(0,6,DCOLS); scrstr("27 ");
videocursor(0,7,DCOLS); scrstr("28 ");
videocursor(0,8,DCOLS); scrstr("29 ");
videocursor(0,9,DCOLS); scrstr("30 ");
videocursor(0,10,DCOLS); scrstr("31 ");
videocursor(0,11,DCOLS); scrstr("32 ");
videocursor(0,12,DCOLS); scrstr("33 ");
videocursor(0,13,DCOLS); scrstr("34 ");
videocursor(0,14,DCOLS); scrstr("35 ");
videocursor(0,15,DCOLS); scrstr("36 ");
videocursor(0,16,DCOLS); scrstr("37 ");
videocursor(0,17,DCOLS); scrstr("38 ");
videocursor(0,18,DCOLS); scrstr("39 ");
videocursor(0,19,DCOLS); scrstr("40 ");
videocursor(0,21,0); scrstr("move cursor with arrows, select with <cr>\n");
videocursor(0,22,0); scrstr("position at top with <, bottom with >.\n");
  ii=DROWS; jj=DCOLS; c='!'; /* ii = max # rows, jj = second column */
  do {
    videocursor(0,di,dj);
    while (!kbdst()) videocursor(0,di,dj);
    c=kbdin();
      switch (c) {
      case '\013': case '\310': if (di==0) di=ii-1; else di--; break; /* up arrow */
      case '\012': case '\320': if (di==ii-1) di=0; else di++; break; /* down arrow */
      case '\014': case '\315': case ' ': dj=(dj==1?jj:1); break;     /* forward */
      case '\010': case '\313': dj=(dj==1?jj:1); break;		      /* backward */
      case '<': di=0; break;					      /* home */
      case '>': di=ii-1; break;					      /* bottom */
      default: break;
      }
    } while (c!='\015');
  videomode(T80X25);
  return(di+(dj==1?0:DROWS)+1);
}

/* edit the rule */
edrule() {char c;
  while (0<1) {
    cscrrul();
    videocursor(0,3,ru);
    c=kbdin();
    if (c=='\015') break;				/* carriage return exits */
    switch (c) {
    case '0': case '1': case '2': case '3': case '4':	/* state */
      ascrule[ru0][ru1][ru2]=c;
      videocattr(0,c,3,1);
      rupl();
      break;
    case '\011':					/* tab = next quad */
      ru2=0; ru1=0; ru0++;
      if (ru0==KK) ru0=0;
      ru=KK*KK*ru0+6;
      break;
    case ' ': case '\315': rupl(); break;		/* space = advance */
    case '\010': case '\313': rumi(); break;		/* backspace */
    case '\013': case '\310': auxrule[ru0][ru1][ru2]='+'; break; /* up arrow */
    case '\012': case '\320': auxrule[ru0][ru1][ru2]=' '; break; /* down arrow */
    default: break;
    }
  }
}

/* advance the rule cursor */
rupl() {
  ru2++;
  if (ru2==KK) {ru2=0; ru1++;}
  if (ru1==KK) {ru1=0; ru0++;}
  if (ru0==KK) {ru0=0; }
  ru=KK*(KK*ru0+ru1)+ru2+6;
  }

/* retract the rule cursor */
rumi() {
  if (ru2!=0) ru2--; else {ru2=KK-1;
  if (ru1!=0) ru1--; else {ru1=KK-1;
  if (ru0!=0) ru0--; else {ru0=KK-1;
     }}}
  ru=KK*(KK*ru0+ru1)+ru2+6;
  }

/* edit totalistic rule */
edtrule() {char c; int  i, j;
  i=0;
  while (i<TS) {
    c=trule[i];
    videocursor(0,TRR,TRC+i);
    videocattr(0,c,3,1);
    c=kbdin();
    if (c=='\015') break;
    switch (c) {
      case '0': case '1': case '2': case '3': case '4':	/* state */
        trule[i]=c;
        videocattr(0,c,2,1);
        i++; break;
      case ' ': case '\315':				/* space = advance */
        i++; break;
      case '\010': case '\313':				/* backspace */
        if (i!=0) i--;
        break;
      case 'x':						/* random rule */
	i=rand();
	for (j=0; j<TS; j++) {
	  if (i==0) i=rand();
	  trule[j]='0'+i%KK;
	  i/=KK;
	  }
	i=0;
	tmenu();
	break;
      default: break;
      }
    }
}

/* edit the line of cells */
edline(nr,nc) int nr, nc; {
char c;
int  i, j, jj, k, l, m, s, sa, sb, t;
int  xx[RR+1], pd[KK*KK*KK+1][2];
int  ll;					/* line to be refreshed */

  videocursor(0,LROW+9,0); scrstr("insert states with 0,1,2,3,4; move cursor");
  videocursor(0,LROW+10,0); scrstr("with keyboard arrows, space or backspace.");
  videocursor(0,LROW+11,0); scrstr("carriage return exits");
  while (0<1) {
  cscrrul();
  s=0; t=0;
  m=0; /* refresh trigger */
  ll=lj;
  videocursor(0,lj+LROW,li);
  c=kbdin();
  if (c == '\015') {videoscroll(LROW+9,0,LROW+12,70,0,3); break;}
  switch (c) {
  case '0': case '1': case '2': case '3': case '4':	/* enter state */
    arr1[nc*lj+li]=c-'0';
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\012': case '\320':				/* downwards */
    lj++; if (lj>=nr) lj=0;
    break;
  case '\013': case '\310':				/* upwards */
    if (lj>0) lj--; else lj=nr-1;
    break;
  case '\014': case '\315': case ' ':			/* forward */
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\010': case '\313':				/* backward */
    if (li>0) li--;
      else {li=nc-1; if (lj>0) lj--; else lj=nr-1;}
    break;
  case '<':						/* left margin */
    li=0;
    break;
  case '>':						/* right margin */
    li=nc-1;
    break;
  case 'z':						/* clear single row */
    for (k=0; k<nc; k++) arr1[nc*lj+k]=0;
    break;
  case 'Z':						/* clear full array */
    for (k=0; k<AL; k++) arr1[k]=0;
    lmenu();
    break;
  case '^':						/* copy line above */
    if (lj>0) for (k=nc*lj; k<nc*(lj+1); k++) arr1[k]=arr1[k-nc];
    lj++; if (lj>=nr) lj=0;
    break;
  case 'x':						/* unmark transitions */
    auxblnk();
    cscrrul();
    break;				
  case 'q':						/* clean up the area */
    ulmenu(3);
    break;
  case '=':						/* insert transition */
    if (lj==0) break;
    linix(xx,li,lj,nc);
    c=xx[RR]+'0';
    ascrule[xx[0]][xx[1]][xx[2]]=c;
    videocursor(0,lj+LROW,li); videocattr(0,c=='0'?'.':c,3,1);
    m=1;
    break;
  case '+':		/* set marker for transition in auxrule */
    if (lj==0) break;
    linix(xx,li,lj,nc);
    c=xx[RR]+'0';
    auxrule[xx[0]][xx[1]][xx[2]]='+';
    videocursor(0,lj+LROW,li); videocattr(0,c=='0'?'.':c,3,1);
    m=1;
    break;
  case '-':		/* remove marker for transition from auxrule */
    if (lj==0) break;
    linix(xx,li,lj,nc);
    c=xx[RR]+'0';
    auxrule[xx[0]][xx[1]][xx[2]]=' ';
    videocursor(0,lj+LROW,li); videocattr(0,c=='0'?'.':c,3,1);
    break;
  case '*':		/* insert transitions for the whole line */
    if (lj==0) break;
    for (i=0; i<nc; i++) {
      linix(xx,i,lj,nc);
      c=xx[RR]+'0';
      ascrule[xx[0]][xx[1]][xx[2]]=c;
      auxrule[xx[0]][xx[1]][xx[2]]='+';
      videocattr(0,c=='0'?'.':c,3,1);
      }
    videocursor(0,lj+LROW,li);
    m=1;
    break;
  case '.':		/* point evolution */
    if (lj==0) break;
    linix(xx,li,lj,nc);
    arr1[li+nc*lj]=ascrule[xx[0]][xx[1]][xx[2]]-'0';
    break;
  case '?':		/* evolution of the whole line */
    if (lj==nr-1) {for (i=0; i<AL-nc; i++) arr1[i]=arr1[i+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      linix(xx,i,lj,nc);
      arr1[i+nc*lj]=ascrule[xx[0]][xx[1]][xx[2]]-'0';
      }
    break;
  case '/':		/* conditional evolution */
    if (lj==nr-1) {for (k=0; k<AL-nc; k++) arr1[k]=arr1[k+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      j=0;
      linix(xx,i,lj,nc);
      for (k=0; k<RR; k++) if (xx[k]>KK-1) j=KK;
      if (auxrule[xx[0]][xx[1]][xx[2]]==' ') j=KK;
      if (j==0) j=ascrule[xx[0]][xx[1]][xx[2]]-'0';
      arr1[i+nc*lj]=j;
      }
    break;
  case 'c':		/* test consistency everywhere */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr; j++) {
    for (i=0; i<nc; i++) {
      linix(xx,i,j,nc);
      k=xx[RR]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,k=='0'?'.':k,k==ascrule[xx[0]][xx[1]][xx[2]]?7:4,1);
      }}
    m=1;
    break;
  case 'C':		/* test consistency for marked neighborhoods */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      linix(xx,i,j,nc);
      k=xx[RR]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,k=='0'?'.':k,
        (k==ascrule[xx[0]][xx[1]][xx[2]] && 
		auxrule[xx[0]][xx[1]][xx[2]]!=' ')?7:4,1);
      }}
    m=1;
    break;
  case 'm':		/* try to match a given evolution */
    videocursor(0,23,10); printf("           ");
    s=0; sa=0; sb=0; t=0;
    if (lj==0) break;
/* in each generation -- that is, on each line -- set all unmarked	*/
/* transitions to zero and push the location of the corresponding cell	*/
    if (lj>1) {
      for (j=1; j<lj; j++) {
      for (i=0; i<nc; i++) {
        linix(xx,i,j,nc);
        if (auxrule[xx[0]][xx[1]][xx[2]]==' ') {
          ascrule[xx[0]][xx[1]][xx[2]]='0';
          auxrule[xx[0]][xx[1]][xx[2]]='+';
          s++; pd[s][0]=i; pd[s][1]=j;}
        arr1[i+nc*j]=ascrule[xx[0]][xx[1]][xx[2]]-'0';
        }}
      }
    cscrrul();
/* Until the target line evolves from the tentative evolution,	*/
/* increment the last recorded transition and repeat the trial.	*/
/* But first, test the target line.				*/
    do {t=0;
    sa=s;
    for (i=k=0; i<nc; i++) {
      linix(xx,i,lj,nc);
      c=xx[RR]+'0';
      if (ascrule[xx[0]][xx[1]][xx[2]]==c &&
	  auxrule[xx[0]][xx[1]][xx[2]]!=' ') continue;
      if (auxrule[xx[0]][xx[1]][xx[2]]==' ') {
          ascrule[xx[0]][xx[1]][xx[2]]=c;
          auxrule[xx[0]][xx[1]][xx[2]]='+';
          s++; pd[s][0]=i; pd[s][1]=lj; continue;}
      k=1;
      }
    sb=s;
    if (k==0) {t=1; break;}
/* undo last minute definitions, which are otherwise kept.	*/
    while (pd[s][1]==lj) {
      linix(xx,pd[s][0],pd[s][1],nc);
      auxrule[xx[0]][xx[1]][xx[2]]=' ';
      ascrule[xx[0]][xx[1]][xx[2]]='0';
      s--;}
/* Given a mismatch, start backtracking; as far as necessary to	*/
/* a cell which has not run through its cycle.			*/
    if (s==0) {t=2; break;}
    do {
      i=pd[s][0]; j=pd[s][1];
      linix(xx,i,j,nc);
      c=ascrule[xx[0]][xx[1]][xx[2]];
      c+=1; k=c-'0';
      ascrule[xx[0]][xx[1]][xx[2]]=c;
      auxrule[xx[0]][xx[1]][xx[2]]='+';	/*---*/
      arr1[i+nc*j]=k;
      if (k<KK) break;
      auxrule[xx[0]][xx[1]][xx[2]]=' ';
      ascrule[xx[0]][xx[1]][xx[2]]='0';
      arr1[i+nc*j]=0;
      s--;
      } while (s>0);
    if (s==0) {t=2; break;}
/* Once again, run through the trial evolution. First, finish	*/
/* out the line where backtracking stopped.			*/
    for (i=pd[s][0],jj=pd[s][1]; i<nc; i++) {
      linix(xx,i,jj,nc);
      if (auxrule[xx[0]][xx[1]][xx[2]]==' ') {
        ascrule[xx[0]][xx[1]][xx[2]]='0';
        auxrule[xx[0]][xx[1]][xx[2]]='+';
        s++; pd[s][0]=i; pd[s][1]=jj;}
      arr1[i+nc*jj]=ascrule[xx[0]][xx[1]][xx[2]]-'0';
      }
/* Then redo the remaining lines, if any.			*/
    jj++; if (jj<lj) {
    for (j=jj; j<lj; j++) {
    for (i=0;  i<nc; i++) {
      linix(xx,i,j,nc);
      if (auxrule[xx[0]][xx[1]][xx[2]]==' ') {
        ascrule[xx[0]][xx[1]][xx[2]]='0';
        auxrule[xx[0]][xx[1]][xx[2]]='+';
        s++; pd[s][0]=i; pd[s][1]=j;}
      arr1[i+nc*j]=ascrule[xx[0]][xx[1]][xx[2]]-'0';
      }} }
/* display progress and test for a waiting interrupt request	*/
    cscrrul();
    lmenu();

    videocursor(0,20,50); printf("(%2d,%2d) %2d %2d",pd[s][1],pd[s][0],sa,sb);
/*    c=kbdin(); if (c=='x') break; */

    if (kbdst()) {kbdin(); t=3; break;}
    } while (t==0);
    videocursor(0,23,10);
    if (t==1) printf("rule found ");
    if (t==2) printf("impossible ");
    if (t==3) printf("interrupted");
    m=1;
    break;
  default: break;
      } /* end switch */
  if (m==0) for (k=0; k<nc; k++) {
    videocursor(0,ll+LROW,k);
    l=arr1[k+nc*ll]+'0';
    videoputc(l=='0'?'.':l,1);
    }
  }     /* end while */
}       /* end edline */

/* calculate index, or rather the neighbors, in edline. */
linix(xx,i,j,nc) int xx[RR+1], i, j, nc; {int k, x;
  xx[(RR-1)/2]=x=i+nc*(j-1);
  if (i==0) xx[0]=x+nc-1; else xx[0]=x-1;
  if (i==nc-1) xx[2]=x-nc+1; else xx[2]=x+1;
  for (k=0; k<RR; k++) xx[k]=arr1[xx[k]];
  xx[RR]=arr1[x+nc];
}

/* display a screen of evolution */
evolve() {int i, j;
  if (sc==7) {xevolve(); return;}
  videomode(COLGRAF);					/* erase the screen */
  videocursor(0,0,0);					/* top text line */
  scrstr(":");
  scrrul(0);
  asctobin();
  for (j=8; j<200; j++) videodot(j,AL-1,2);
  for (j=8; j<200; j++) {				/* evolve for 192 generations */
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
    if (kbdst()) {kbdin(); break;}
    }
}

/* display a Plantronic screen of evolution */
xevolve() {int i, j;
  videomode(COLGRAF);					/* erase the screen */
  outportb(0x3dd,0x10);
  videocursor(0,0,0);					/* top text line */
  scrstr(":");
  hscrrul();
  asctobin();
  for (j=8; j<200; j++) viddot(j,AL-1,2);
  for (j=8; j<200; j++) {				/* evolve for 192 generations */
    for (i=0; i<AL; i++) viddot(j,i,arr1[i]);
    onegen(AL);
    if (kbdst()) {kbdin(); break;}
    }
}

/* Plantronic videodot */
viddot(y,x,l) int x, y, l; {int a, b, c, cc, d, dd, s, u, v, w, z;
  if (l==1) l=5;
  if (l==2) l=6;
  if (l==3) l=12;
  if (l==4) l=10;
  a=256*(l%4);
  b=256*((l/4)&0x3);
  u=x/4;
  v=x%4;
  w=2*(y/2);
  z=u+40*w;
  s=2*(3-v);
  if (y%2==0) c=peekb(0xb800,z); else c=peekb(0xba00,z);
  if (y%2==0) d=peekb(0xbc00,z); else d=peekb(0xbe00,z);
  c*=256; d*=256;
  cc=(((c>>s)&0xFCFF)|a)<<s;
  dd=(((d>>s)&0xFCFF)|b)<<s;
  cc/=256; dd/=256;
  if (y%2==0) pokeb(0xb800,z,cc); else pokeb(0xba00,z,cc);
  if (y%2==0) pokeb(0xbc00,z,dd); else pokeb(0xbe00,z,dd);
}

/* copy ascrule over to binrul */
asctobin() {int i0, i1, i2;
  for (i0=0; i0<KK; i0++) {
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
    binrule[i0][i1][i2]=ascrule[i0][i1][i2]-'0';
    }}}
}

/* evolution for one generation */
onegen(j) int j; {int i;
  if (j<2) return;
  arr2[0]=binrule[arr1[j-1]][arr1[0]][arr1[1]];
  for (i=1; i<j-1; i++) arr2[i]=binrule[arr1[i-1]][arr1[i]][arr1[i+1]];
  arr2[j-1]=binrule[arr1[j-2]][arr1[j-1]][arr1[0]];
  for (i=0; i<j; i++) arr1[i]=arr2[i];
}

/* map the automaton state into the unit square */
pprob() {int i, j; float x, y, z, k;
i=AL/2; k=(float)KK; z=(float)((197/(KK*KK))*KK*KK);
x=0.0; for (j=1; j<30; j++) x=(x+(float)arr1[i+j])/k;
y=0.0; for (j=1; j<30; j++) y=(y+(float)arr1[i-j])/k;
x=(x+(float)arr1[i])/k;
videodot((int)(z*(1.0-y)),(int)(z*x),1);
}

/* set up a grid for the unit square */
pgrid() {int i, j, k, l;
k=KK*KK+1;
l=195/(k-1);
for (i=0; i<k; i++) {
for (j=0; j<k; j++) {
videodot(l*i,l*j,2);
}}
}

/* tutorial and Help screen */
tuto() {
    videomode(T80X25);
    videocursor(0,2,0);
    scrstr("<Copyright (C) 1987, 1988, 1989 - H.V.McIntosh, G.Cisneros S.>");
    videocursor(0,4,0);
    scrstr("             *** LIFE in One Dimension ***");
    videocursor(0,6,0);
    scrstr("Five States - Black(0), Cyan(1), Magenta(2), White(3), X(4).");
    videocursor(0,8,0);
    scrstr("First neighbors - one on each side, three altogether.");
    videocursor(0,10,0);
    scrstr("Complete transition rule - random, edited, or stored.");
    videocursor(0,12,0);
    scrstr("Totalistic transition rule - random, edited, or stored.");
    videocursor(0,14,0);
    scrstr("Initial Cellular Array - random, edited, or patterned.");
    videocursor(0,16,0);
    scrstr("Submenus in options t(probabilities) and d(de Bruijn)");
    videocursor(0,17,0);
    scrstr("will be displayed in response to typing ?");
    videocursor(0,19,0);
    scrstr("Use any key to terminate a display in progress.");
    videocursor(0,22,0);
    scrstr("now, ... press any key to continue.");
}

/* rule menu */
rmenu() {
    videocursor(0,0,0); scrstr("   0: "); scrrul(0);
    videocursor(0,1,0); scrstr("   1: "); scrrul(1);
    videocursor(0,2,0); scrstr("   2: "); scrrul(2);
    videocursor(0,3,0); scrstr("   3: "); scrrul(3);
    videocursor(0,4,0); scrstr("   4: "); scrrul(4);
    if (istot()==1) {videocursor(0,TRR,TRC); xmenu(totonu(0));}
    videocursor(0,5,0);
    scrstr("    r(rule), l(line), #nn(stored), g(graph), q(quit), t(prob),");
    videocursor(0,6,0);
    scrstr("           x(rand rule), y(rand line), u(unit line), d(deBruijn)");
    videocursor(0,7,0);
    scrstr("           @nn(tot/rule), $nn(12 tot/rules), T(ed tot/rule).");
    videocursor(0,5,0);
    }

/* totalistic rule menu*/
tmenu() {
    videocursor(0,TRR-2,TRC-6);
    scrstr("      totalistic rule");
    videocursor(0,TRR-1,TRC-6);
    scrstr("      0..1..6..9..C");
    videocursor(0,TRR,TRC-6);
    scrstr("rule: ");
    tscrrul();
    videocursor(0,TRR,TRC);
    }

/* plain line menu */
lmenu() {int i, j; char c;
  for (j=0; SL*j<AL; j++) {videocursor(0,9+j,0);
  for (i=0; i<SL; i++) {c='0'+arr1[SL*j+i]; videoputc(c=='0'?'.':c,1); } }
  videocursor(0,5,0); }

/* uniform line menu */
ulmenu(l) int l; {int i, j, k;
  for (j=0; j*SL<AL; j++) {
  for (i=0; i<SL;    i++) {
    videocursor(0,9+j,i);
    k=arr1[j*SL+i];
    videocattr(0,k==0?'.':k+'0',l,1);
    }}
  }

/* color line menu */
clmenu() {int c, i, j;
  for (j=0; SL*j<AL; j++) {
    for (i=0; i<SL; i++) {
      videocursor(0,9+j,i);
      c=arr1[SL*j+i];
      videocattr(0,c==0?'.':'0'+c,color(c),1);
      }}
  videocursor(0,5,0); }

/* get compatible color */
int color(i) int i; {
  switch (i) {
    case 0:  return 2;
    case 1:  return 3;
    case 2:  return 5;
    case 3:  return 7;
    case 4:  return 4;
    default: return 6;
}}

/* display rule number */
xmenu(n) int n; {
char xx[6];
int i, nn;  
    nn=sprintf(xx,"%5u",n);
    videocursor(0,XRR,XRC);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,XRR,XRC); }

/* clear screen space for rule number */
xblnk() {videocursor(0,XRR,XRC); scrstr("     "); videocursor(0,5,0); }

/* copy saved rule #n into active rule */
xtoasc(n) int n; {int i;
  xmenu(n+1);
  for (i=0; i<TS; i++) trule[i]=ixrule[n][i];
  totoasc();
 }

/* change totalistic rule to general rule */
totoasc() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
  ascrule[i0][i1][i2]=trule[i0+i1+i2];
}

/* change decimal totalistic rule to sum values */
nutoto(x) int x; {int i;
for (i=0; i<TS; i++) {trule[i]=x%KK+'0'; x/=KK;}}

/* change sum values to decimal totalistic rule */
int totonu(i) int i; {int r;
  if (i<TS) r=(trule[i]-'0')+KK*totonu(i+1); else r=0;
  return r; }

/* test whether a rule is totalistic */
int istot() {int i0, i1, i2, l;
    l=1;
    trule[0]=ascrule[0][0][0];
    trule[1]=ascrule[1][0][0];
    trule[2]=ascrule[2][0][0];
    trule[3]=ascrule[3][0][0];
    trule[4]=ascrule[4][0][0];
    trule[5]=ascrule[4][1][0];
    trule[6]=ascrule[4][2][0];
    trule[7]=ascrule[4][3][0];
    trule[8]=ascrule[4][4][0];
    trule[9]=ascrule[4][4][1];
    trule[10]=ascrule[4][4][2];
    trule[11]=ascrule[4][4][3];
    trule[12]=ascrule[4][4][4];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
      if (ascrule[i0][i1][i2]!=trule[i0+i1+i2]) l=0;
    return l; }

/* generate a random line of cells in arr1 */
ranlin() {int i, c;
for (i=0; i<AL; i++) {
  if (i%7 == 0) c=rand();
  arr1[i]=c%KK; c/=KK;}
}

/* clear auxrule to blanks */
auxblnk() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
  auxrule[i0][i1][i2]=' ';
}

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}

/* display the rule number on the screen */
scrrul(i1) int i1; {int i0, i2;
  for (i0=0; i0<KK; i0++) {
  for (i2=0; i2<KK; i2++) {
  videoputc(ascrule[i0][i1][i2],1);
  }} }

/* display the rule number on the screen in glorious technicolor */
cscrrul() {int i, i0, i1, i2;
  i=6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
    {
    videocursor(0,3,i++);
    videocattr(0,ascrule[i0][i1][i2],auxrule[i0][i1][i2]==' '?3:6,1);
    }
  }

/* display the hexadecimal rule number on the screen */
hscrrul() {char i; int i0, i1, i2;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2+=2)
    {
    i=ascrule[i0][i1][i2]+4*(ascrule[i0][i1][i2+1]-'0');
    if (i>'9') i=i+('A'-':');
    videoputc(i,1);      
    }
  }

/* display totalistic rule number by sum */
tscrrul() {int i;
  for (i=0; i<TS; i++) {videoputc(trule[i],1); } }

/* change decimal totalistic rule to sum values */
nutowo1(x) int x; {int i;
for (i=0; i<TS; i++) {prule1[i]=x%2+'0'; x/=2;}}

/* change Wolfram rule number decimal rule number */
int wotonu1(i) int i; {int r;
  if (i<TS) r=(prule1[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* change decimal number to Wolfram rule number */
nutowo2(x) int x; {int i;
for (i=0; i<TS; i++) {prule2[i]=x%2+'0'; x/=2;}}

/* change sum values to decimal totalistic rule */
int wotonu2(i) int i; {int r;
  if (i<TS) r=(prule2[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* write a string in graphics mode */
scrstr(s) char *s;
  {for (; *s != '\0'; s++) videoputc(*s,1); }

/* keyboard status */
int kbdst() {return(bdos(11) & 0xFF);}

/* direct keyboard input, no echo */
kbdin() {int c;
  if ((c = bdos(7) & 0xFF) == '\0') c = (bdos(7) & 0xFF) | 0x80;
  return(c);
}

/* read number from keyboard */
int numin(n) int n; {char c;
  c=kbdin();
  if (c>='0'&&c<='9') return(numin(10*n+(c-'0'))); else return(n);
}

/* end */
