
/* nxprob4h.c						*/
/* program for lcau4h.c option 't'			*/
/* calculate probabilities related to evolution		*/
/* Harold V. McIntosh, 10 August 1987			*/
/* 4 September 1987 - tetrahedral stereopair		*/
/* 30 December 1987 - contoured stereopair layers	*/
/* 3 April 1989 - n, N for totalistic statistics	*/
/* 15 September 1989 - Entropy calculations		*/
/* 5 October 1989 - adapted to r = 1/2			*/
/* 30 April 1990 - subser and equivalence indices	*/

/* references:							*/
/*								*/
/* W. John Wilbur, David J. Lipman and Shihab A. Shamma		*/
/* On the prediction of local patterns in cellular automata	*/
/* Physica 19D 397-410 (1986)					*/ 
/*								*/
/* Howard A. Gutowitz, Jonathan D. Victor and Bruce W. Knight	*/
/* Local structure theory for cellular automata			*/
/* Physica 28D 18-48 (1987)					*/

/*	Copyright (C) 1987	*/
/*	Copyright (C) 1988	*/
/*	Copyright (C) 1989	*/
/*	Copyright (C) 1990	*/
/*	Harold V. McIntosh	*/

# include <math.h>
# include "nxlcau6h.h"
# include "nxprob6h.h"
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>


extern char   ascrule[KK][KK];			/* ASCII transition values */
extern int    binrule[KK][KK];			/* binary transition values */
extern char    arr1[AL], arr2[AL];		/* line of cells */
extern double wmul[KK], wmvl[KK];		/* left mass point */
extern double wmur[KK], wmvr[KK];		/* right mass point */

# define BER	    0	/* row (from bottom) for Bernstein p.	*/
# define BEC	  205	/* column for Bernstein Polynomial	*/
# define BROW	   14   /* row for bar charts		*/
# define EROW	   23   /* row for evolution synopsis	*/

/* edit the probability screen	*/

edtri() {char c; int i;
  videomode(COLGRAF);
  videopalette(YELREGR);

  videocursor(0,1,33);  videoputc('2',3);
  videocursor(0,13,23); videoputc('0',3);
  videocursor(0,13,39); videoputc('1',3);

  tbase();
  edges(50,3);

  while (0<1) {
  videocursor(0,0,0);  hscrrul();
  videocursor(0,0,36); videoputc('?',2);
  c=kbdin();
  if (c == '\015') break;
  videocursor(0,0,38); videoputc(c,2);
  videocursor(0,0,36); videoputc(' ',0);
  switch (c) {
    case '+': videopalette(WHCYMAG); break;
    case '-': videopalette(YELREGR); break;
    case 'a': nblclr(); asfreq(3); break;
    case 'b': bpanel(); break;
    case 'B': tbase(); break;
    case 'e': pevolve(); break;
    case 'E': tedge(); break;
    case 'g': for (i=0, asctobin(); i<50; i++, onegen(AL)) lifreq(2); break;
    case 'G': for (i=0, asctobin(); i<200; i++, onegen(AL)) lifreq(1); break;
    case 'h': edges(80,3); break;
    case 'k': videohegrid(10,0); break;
    case 'l': lpanel(); break;
    case 'm': nblclr(); moncar(1,1); break;
    case 'M': nblclr(); moncar(50,2); break;
    case 'n': nblclr(); neisum(8*BROW,AL,48); break;
    case 'N': nblclr(); nbrsum(8*BROW,AL,48); break;
    case 't': hpdiff(12,8); break;
    case 'p': elayer(40,0); break;
    case 'q': elayer(40,1); break;
    case 'Q': eqrndx(); break;
    case 'r': elayer(40,2); break;
    case 's': elayer(40,3); break;
    case 'S': subndx(); break;
    case 'u': elayer(72,8); break;
    case 'v': olayer(72,8); break;
    case 'U': elayer(42,8); break;
    case 'V': olayer(42,8); break;
    case '1': nblclr(); oneblfreq(8*BROW,300,56); break;
    case '2': nblclr(); twoblfreq(8*BROW,300,56); break;
    case '3': nblclr(); thrblfreq(8*BROW,300,56); break;
    case '4': nblclr(); foublfreq(8*BROW,300,56); break;
    case '5': nblclr(); fivblfreq(8*BROW,300,56); break;
    case 'z': nblclr(); break;
    case '/': videomode(COLGRAF); videopalette(YELREGR); break;
    case '!': nblclr(); pamenu(); break;
    case '?': nblclr(); trmenu(); break;
    default: break;
    } /* end switch */
  }   /* end while  */
  videopalette(WHCYMAG);
  videomode(T80X25);
}     /* end edtri  */

/* show option menu */

/* show n-block probabilities */
void nblcur(n) int n; {switch (n) {
    case '1': oneblfreq(0,300,48); break;
    case '2': twoblfreq(0,300,48); break;
    case '3': thrblfreq(0,300,48); break;
    case '4': foublfreq(0,300,48); break;
    case '5': fivblfreq(0,300,48); break;
//    case '6': sixblfreq(0,300,48); break;
//    case '7': svnblfreq(0,300,48); break;
    default: break;}
}

trmenu() {
  videocursor(0,BROW,0);
  printf("a     - a priori estimates, entropy\n");
  printf("m,g,G - sample evolution\n");
  printf("pqrs  - prob of states\n");
  printf("t,u,v - sq, odd-even\n");
  printf("12345 - n-block bar charts\n");
  printf("n,N   - totalistic statistics\n");
  printf("e     - 16 lines evolution\n");
  printf("z/?!  - clear panel, screen; show menu\n");
  printf("<cr>  - exit\n");
}

/* show parameter menu */

pamenu() {
  videocursor(0,BROW,0);
  printf("B     - tetrahedron on face           \n");
  printf("E     - tetrahedron on edge           \n");
  printf("h     - draw edges                    \n");
  printf("k     - draw grid                     \n");
  printf("      -                               \n");
  printf("+-    - change color quad             \n");
  printf("      -                               \n");
  printf("z/?!  - clear panel, screen; show menu\n");
  printf("<cr>  - exit\n");
}

/* show sixteen lines of evolution on screen */

pevolve() {int i, j;
  videoscroll(EROW,0,EROW+1,40,0,0);
  asctobin();
  for (j=8*EROW; j<8*EROW+16; j++) {
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
    }
}

/* Clear a space for the n-block frequencies */
nblclr() {videoscroll(BROW,0,BROW+8,40,0,0);}

/* locations of the weighted masses */
/* for tetrahedron sitting on base  */

tbase() {
  wmul[0]=160.0; wmvl[0]=100.0;
  wmul[1]=310.0; wmvl[1]=100.0;
  wmul[2]=240.0; wmvl[2]=195.0;
  wmul[3]=250.0; wmvl[3]=157.0;

  wmur[0]= 10.0; wmvr[0]=100.0;
  wmur[1]=160.0; wmvr[1]=100.0;
  wmur[2]= 90.0; wmvr[2]=195.0;
  wmur[3]= 80.0; wmvr[3]=157.0;
}

/* locations of the weighted masses */
/* for tetrahedron standing on edge */

tedge() {
  wmul[0]=160.0; wmvl[0]=157.0;
  wmul[1]=310.0; wmvl[1]=157.0;
  wmul[2]=240.0; wmvl[2]=195.0;
  wmul[3]=250.0; wmvl[3]=100.0;

  wmur[0]= 10.0; wmvr[0]=157.0;
  wmur[1]=160.0; wmvr[1]=157.0;
  wmur[2]= 90.0; wmvr[2]=195.0;
  wmur[3]= 80.0; wmvr[3]=100.0;
}


/* bdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
bdot(i,j,l) int i, j, l; {videodot(199-BER-j,BEC+i,l);}

/* pdot(i,j,l) - place polkadot on Bernstein screen at (i,j) in color l */
pdot(i,j,l) int i, j, l; {videodot(199-BER-j,BEC+i,l+i%2);}

/* plot a single point on a triangular grid */

videotrdot(u,v,x,l) double x[KK]; int u, v, l; {double s, y, z;
s=199.0/(x[0]+x[1]+x[2]); y=x[1]*s; z=x[2]*s;
videodot(u-(int)(0.433*z),v+(int)(0.250*(y+y+z)),l);
}

/* plot a single point on a tetrahedral grid */

videohedot(a,b,p,l) double p[KK]; int a, b, l; {double s, u, v, w;
s=199.0/(p[0]+p[1]+p[2]+p[3]);
u=(p[1]+p[3])*s; v=p[2]*s; w=(p[1]-p[3])*s;
videodot(a-(int)(0.433*p[1]*s),b-50+(int)(0.250*(u+u+v)),l);
videodot(a-(int)(0.433*p[1]*s),b+50+(int)(0.250*(w+w+v)),l);
}

/* plot a stereo point on a triangular grid */
/* (u,v) screen location lower left corner  */
/* (x,y,z) triangular coordinates	    */
/* t       height			    */
/* l       color			    */

videostdot(u,v,x,y,z,t,l) double x, y, z, t; int u, v, l; {double s;
s=199.0/(x+y+z); x=x*s; y=y*s; z=z*s;
videodot(u-(int)(0.433*z),v+(int)(0.250*(y+y+z+t)),l);
}

/* set up a triangular gridwork */
/* (u,v) = corner of triangle	*/
/* n = number of rulings	*/
/* l = color of lines		*/

videotrgrid(u,v,n,l) int u, v, n, l; {
int i, j;
double a[KK];

if(n==0) return;
for (i=0; i<=n;  i++) {
for (j=0; i+j<=n; j++) {
  a[0]=((double)(i)/(double)(n));
  a[1]=((double)(j)/(double)(n));
  a[2]=1.0-a[0]-a[1];
  videotrdot(u,v,a,l);
  }}
}

/* display state frequencies in arr1 as a dot in Bezier diagram */

lifreq(l) int l; {int i, stat[KK]; double staf[KK], s;
s=1.0/((double)(AL));
for (i=0; i<KK; i++) stat[i]=0;
for (i=0; i<AL; i++) (stat[arr1[i]])++;
for (i=0; i<KK; i++) staf[i]=s*((double)(stat[i]));
videomassdot(staf,l);
}

/* plot a single point as a weighted mass center */

videomassdot(x,l) double *x; int l; {double ul, vl, ur, vr; int i;
if (l>=KK) return;
for (i=0, ul=0.0; i<KK; i++) ul+=x[i]*wmul[i];
for (i=0, vl=0.0; i<KK; i++) vl+=x[i]*wmvl[i];
for (i=0, ur=0.0; i<KK; i++) ur+=x[i]*wmur[i];
for (i=0, vr=0.0; i<KK; i++) vr+=x[i]*wmvr[i];
videodot(199-(int)vl,(int)ul,l);
videodot(199-(int)vr,(int)ur,l);
}

/* plot a point in 4 triangles - face projections of a tetrahedron */

video4dots(x,l) int l; double x[KK]; {double u[KK-1];
u[0]=x[0]; u[1]=x[1]; u[2]=x[2]; videotrdot(99,200,u,l);
u[0]=x[1]; u[1]=x[2]; u[2]=x[3]; videotrdot(199,0,u,l);
u[0]=x[2]; u[1]=x[3]; u[2]=x[0]; videotrdot(149,100,u,l);
u[0]=x[3]; u[1]=x[0]; u[2]=x[1]; videotrdot(199,200,u,l);
}

/* set up a triangular gridwork of edge n in color l */

videohegrid(n,l) int n, l; {
int i0, i1, i2;
double x[KK], nn;

if (n==0) return;
nn=1.0/((double)(n));
for (i0=0, x[0]=0.0; i0<=n; i0++, x[0]+=nn) {
for (i1=0, x[1]=0.0; i0+i1<=n; i1++, x[1]+=nn) {
for (i2=0, x[2]=0.0; i0+i1+i2<=n; i2++, x[2]+=nn) {
  x[3]=1.0-x[0]-x[1]-x[2];
  videomassdot(x,l);
  }}}
}

/* show the skeleton framework of the tetrahedron */
/* n = points per edge				  */
/* l = color of edge				  */

edges(n,l) int n, l; {int i; double x[KK], nn;
if (n==0) return;
nn=1.0/((double)(n));
for (i=0, uvec(x,0); i<=n; i++, x[0]-=nn, x[1]+=nn) videomassdot(x,l);
for (i=0, uvec(x,0); i<=n; i++, x[0]-=nn, x[2]+=nn) videomassdot(x,l);
for (i=0, uvec(x,0); i<=n; i++, x[0]-=nn, x[3]+=nn) videomassdot(x,l);
for (i=0, uvec(x,1); i<=n; i++, x[1]-=nn, x[2]+=nn) videomassdot(x,l);
for (i=0, uvec(x,1); i<=n; i++, x[1]-=nn, x[3]+=nn) videomassdot(x,l);
for (i=0, uvec(x,2); i<=n; i++, x[2]-=nn, x[3]+=nn) videomassdot(x,l);
}

/* unit vector */

uvec(x,i) double *x; int i; {int j;
for (j=0; j<KK; j++) x[j]=0.0;
x[i]=1.0;
}

/* homogeneous vector */

hvec(x) double *x; {int i; double s;
s=1.0/((double)(KK));
for (i=0; i<KK; i++) x[i]=s;
}

/* layered contour map in stereoscopic tetrahedron */
/* n = length of edge  */
/* l = function option */

elayer(n,l) int n, l; {int m, nn;
  m=n/4; nn=4*m;
  pdiff(3*m,nn,l);
  pdiff(2*m,nn,l);
  pdiff(1*m,nn,l);
  pdiff(0*m,nn,l);
  edges(nn,3);
}

/* layered contour map in stereoscopic tetrahedron */
/* n = length of edge  */
/* l = function option */

olayer(n,l) int n, l; {int m, nn;
  m=n/8; nn=8*m;
  pdiff(7*m,nn,l);
  pdiff(5*m,nn,l);
  pdiff(3*m,nn,l);
  pdiff(1*m,nn,l);
  edges(nn,3);
}

/* plot a single line on a triangular grid */

videotrli(a,b,c,l) double a, b, c; int l; {double s, y, z;
s=400.0/(a+b+c); y=b*s; z=c*s;
if (l==0) PSmoveto(0.250*(y+y+z),0.433*z);
if (l==1) PSlineto(0.250*(y+y+z),0.433*z);
}

/* set up a line grid */

linegrid(n) int n; {int i; double p, dp;
dp=1.0/((double) n);
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrli(0.0, p, 1.0-p, 0);
    videotrli(1.0-p, p, 0.0, 1);
    }
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrli(p, 1.0-p, 0.0, 0);
    videotrli(p, 0.0, 1.0-p, 1);
    }
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrli(1.0-p, 0.0, p, 0);
    videotrli(0.0, 1.0-p, p, 1);
    }
  }

/* ------------ */

/* check the indices for all the subsets */
subndx() {
asctobin();
videocursor(0,BROW,0);   printf("Subset indices:");
videocursor(0,BROW+1,0); printf("(0123) 64/64");
thes(); twos(); ones();}

/* check the three-element subsets */
thes() {int frac3();
videocursor(0,BROW+2,0);  printf("(012)  %2d/9",frac3(0,1,2)); 
videocursor(0,BROW+2,20); printf("(013)  %2d/9",frac3(0,1,3)); 
videocursor(0,BROW+3,0);  printf("(023)  %2d/9",frac3(0,2,3)); 
videocursor(0,BROW+3,20); printf("(123)  %2d/9",frac3(1,2,3)); 
}

/* calculate fraction of neighborhoods which conserve the evolution */
/* of a particular three element subset and report this as an index */
int frac3(a,b,c) int a, b, c; {int k, i0, i1, s[3], isabc();
k=0; s[0]=a; s[1]=b; s[2]=c;
for (i0=0; i0<3; i0++)
for (i1=0; i1<3; i1++)
  if (isabc(a,b,c,binrule[s[i0]][s[i1]])) k++;
return k;
}

/* check membership in three-element set */
int isabc(a,b,c,i) int a, b, c, i; {return (i==a||i==b||i==c);}

/* check the two-element subsets */
twos() {int frac2();
videocursor(0,BROW+4,0);  printf("(01)   %2d/ 4",frac2(0,1)); 
videocursor(0,BROW+4,20); printf("(02)   %2d/ 4",frac2(0,2)); 
videocursor(0,BROW+5,0);  printf("(03)   %2d/ 4",frac2(0,3)); 
videocursor(0,BROW+5,20); printf("(12)   %2d/ 4",frac2(1,2)); 
videocursor(0,BROW+6,0);  printf("(13)   %2d/ 4",frac2(1,3)); 
videocursor(0,BROW+6,20); printf("(23)   %2d/ 4",frac2(2,3)); 
}

/* calculate fraction of neighborhoods which conserve the evolution */
/* of a particular two element subset and report this as an index */

int frac2(a,b) int a, b; {int k, i0, i1, s[2], isab();
k=0; s[0]=a; s[1]=b;
for (i0=0; i0<2; i0++)
for (i1=0; i1<2; i1++)
  if (isab(a,b,binrule[s[i0]][s[i1]])) k++;
return k;
}

/* check membership in two-element set */
int isab(a,b,i) int a, b, i; {return (i==a||i==b);}

/* check whether one-element subset is closed */
ones() {int frac1();
videocursor(0,BROW+7,0);  printf("(0)    %2d/ 1",frac1(0)); 
videocursor(0,BROW+7,20); printf("(1)    %2d/ 1",frac1(1)); 
videocursor(0,BROW+8,0);  printf("(2)    %2d/ 1",frac1(2)); 
videocursor(0,BROW+8,20); printf("(3)    %2d/ 1",frac1(3)); 
}

/* calculate fraction of neighborhoods which conserve the evolution */
/* of a particular one element subset and report this as an index */
int frac1(a) int a; {int k; k=0; if (a==binrule[a][a]) k++; return k;}

/* --------------- */

/* calculate equivalence relation indices */

eqrndx() {
int    i, j, i0, i1, i3, i4, q[15], t[15];
double r[15];
asctobin();
for (i=0; i<15; i++) q[i]=t[i]=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
  {
  i=binrule[i0][i1];
  j=binrule[i3][i4];
  q[0]++;
  if (eq01(i0,i3)&&eq01(i1,i4)) {t[1]++; if (eq01(i,j)) q[1]++;}
  if (eq02(i0,i3)&&eq02(i1,i4)) {t[2]++; if (eq02(i,j)) q[2]++;}
  if (eq03(i0,i3)&&eq03(i1,i4)) {t[3]++; if (eq03(i,j)) q[3]++;}
  if (eq04(i0,i3)&&eq04(i1,i4)) {t[4]++; if (eq04(i,j)) q[4]++;}
  if (eq05(i0,i3)&&eq05(i1,i4)) {t[5]++; if (eq05(i,j)) q[5]++;}
  if (eq06(i0,i3)&&eq06(i1,i4)) {t[6]++; if (eq06(i,j)) q[6]++;}
  if (eq07(i0,i3)&&eq07(i1,i4)) {t[7]++; if (eq07(i,j)) q[7]++;}
  if (eq08(i0,i3)&&eq08(i1,i4)) {t[8]++; if (eq08(i,j)) q[8]++;}
  if (eq09(i0,i3)&&eq09(i1,i4)) {t[9]++; if (eq09(i,j)) q[9]++;}
  if (eq10(i0,i3)&&eq10(i1,i4)) {t[10]++; if (eq10(i,j)) q[10]++;}
  if (eq11(i0,i3)&&eq11(i1,i4)) {t[11]++; if (eq11(i,j)) q[11]++;}
  if (eq12(i0,i3)&&eq12(i1,i4)) {t[12]++; if (eq12(i,j)) q[12]++;}
  if (eq13(i0,i3)&&eq13(i1,i4)) {t[13]++; if (eq13(i,j)) q[13]++;}
  q[14]++;
  }
r[0]=1.0;
for (i=1; i<14; i++) r[i]=((double)q[i])/((double)t[i]);
r[14]=1.0;
videocursor(0,BROW+4,20);  printf("Equivalence indices");
videocursor(0,BROW,0);     printf("(0123)       %5.3f",r[0]); 
videocursor(0,BROW+1,0);   printf("(0)(123)     %5.3f",r[1]); 
videocursor(0,BROW+1,20);  printf("(1)(023)     %5.3f",r[2]); 
videocursor(0,BROW+2,0);   printf("(2)(013)     %5.3f",r[3]); 
videocursor(0,BROW+2,20);  printf("(3)(012)     %5.3f",r[4]); 
videocursor(0,BROW+3,0);   printf("(01)(23)     %5.3f",r[5]); 
videocursor(0,BROW+4,0);   printf("(02)(13)     %5.3f",r[6]); 
videocursor(0,BROW+5,0);   printf("(03)(12)     %5.3f",r[7]); 
videocursor(0,BROW+6,0);   printf("(01)(2)(3)   %5.3f",r[8]); 
videocursor(0,BROW+6,20);  printf("(23)(0)(1)   %5.3f",r[9]); 
videocursor(0,BROW+7,0);   printf("(02)(1)(3)   %5.3f",r[10]); 
videocursor(0,BROW+7,20);  printf("(13)(0)(2)   %5.3f",r[11]); 
videocursor(0,BROW+8,0);   printf("(03)(1)(2)   %5.3f",r[12]); 
videocursor(0,BROW+8,20);  printf("(12)(0)(3)   %5.3f",r[13]); 
videocursor(0,BROW,20);    printf("(0)(1)(2)(3) %5.3f",r[14]); 
}

eq01(i,j) int i, j; {return ((i==0&&j==0)||(i!=0&&j!=0));}
eq02(i,j) int i, j; {return ((i==1&&j==1)||(i!=1&&j!=1));}
eq03(i,j) int i, j; {return ((i==2&&j==2)||(i!=2&&j!=2));}
eq04(i,j) int i, j; {return ((i==3&&j==3)||(i!=3&&j!=3));}
eq05(i,j) int i, j; {return (((i==0||i==1)&&(j==0||j==1))||
	((i==2||i==3)&&(j==2||j==3)));}
eq06(i,j) int i, j; {return (((i==0||i==2)&&(j==0||j==2))||
	((i==1||i==3)&&(j==1||j==3)));}
eq07(i,j) int i, j; {return (((i==0||i==3)&&(j==0||j==3))||
	((i==1||i==2)&&(j==1||j==2)));}
eq08(i,j) int i, j; {return (((i==0||i==1)&&(j==0||j==1))||
	(i==2&&j==2)||(i==3&&j==3));}
eq09(i,j) int i, j; {return (((i==2||i==3)&&(j==2||j==3))||
	(i==0&&j==0)||(i==1&&j==1));}
eq10(i,j) int i, j; {return (((i==0||i==2)&&(j==0||j==2))||
	(i==1&&j==1)||(i==3&&j==3));}
eq11(i,j) int i, j; {return (((i==1||i==3)&&(j==1||j==3))||
	(i==0&&j==0)||(i==2&&j==2));}
eq12(i,j) int i, j; {return (((i==0||i==3)&&(j==0||j==3))||
	(i==1&&j==1)||(i==2&&j==2));}
eq13(i,j) int i, j; {return (((i==1||i==2)&&(j==1||j==2))||
	(i==0&&j==0)||(i==3&&j==3));}

/* ------------------ */

/* options 'm', 'M'.				*/
/* "Monte Carlo" determination of probabilities */
/* n = mumber of generations			*/
/* l = color of dot				*/

moncar(n,l) int n, l; {
int i, j, k, b[KK], bb[KK][KK];
double bf[KK], bbf[KK][KK];
double a[KK], aa[KK], d[KK], el, en, t, tt, s;

nblclr();
asctobin();
el=1.0/((double)AL);
en=1.0/((double)n);
for (i=0; i<KK; i++) {a[i]=0.0; aa[i]=0.0;}
for (k=0; k<n; k++) {
  onegen(AL);
  for (i=0; i<KK; i++) b[i]=0;
  for (i=0; i<AL; i++) b[arr1[i]]++;
  for (i=0; i<KK; i++) bf[i]=((double)(b[i]))*el;
  for (i=0; i<KK; i++) {a[i]+=bf[i]; aa[i]+=bf[i]*bf[i];}
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) bb[i][j]=0;
  for (i=1; i<AL; i++) bb[arr1[i-1]][arr1[i]]++;
  bb[arr1[AL-1]][arr1[0]]++; 
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
    bbf[i][j]=((double)(bb[i][j]))*el;
  videomassdot(bf,l);
  }
for (i=0; i<KK; i++) {a[i]*=en; aa[i]*=en;}
videocursor(0,BROW,0);
s=1.0/log((double)KK);
t=0.0; tt=0.0;
for (i=0; i<KK; i++) t+=bf[i]*log(bf[i]);
for (i=0; i<KK; i++) d[i]=sqrt(aa[i]-a[i]*a[i]);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) tt+=bbf[i][j]*log(bbf[i][j]);
printf("lambda       = %5.3f\n",1.0-bf[0]);
printf("entropy      = %5.3f, pairs %5.3f\n",-t*s,-0.5*tt*s);
printf("\n");
printf("frequency    = ");
for (i=0; i<KK; i++) printf("%5.3f ",a[i]);
printf("\n");
if (n>1) {
  printf("deviation   =  ");
  for (i=0; i<KK; i++) printf("%5.3f ",d[i]);
  printf("\n");
  }
printf("pair freq    = ");
for (i=0; i<KK; i++) printf("%5.3f ",bbf[0][i]);
printf("\n");
for (j=1; j<KK; j++) {
  printf("               ");
  for (i=0; i<KK; i++) printf("%5.3f ",bbf[j][i]);
  printf("\n");
  }
}

/* option 'n' (general neighborhood) */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

void gnlsum(ll,mm,nn) int ll, mm, nn; {
int    i0, i1, i ;
int    ii, k, m, n, l, b[KK][KK];
double em, en, f, bf[KK][KK], bft[KK][KK];
printf("hello from gnlsum\n"); 
asctobin();
em=((double)(mm));
// f=1.0;
f=115.0/((double)(AL));
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
	bft[i0][i1]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  for (i=0; i<mm; i++) printf(" arr1[%d]=%d\n",i,arr1[i]);
  gnrsum(mm,b);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
	bf[i0][i1]=((double)(b[i0][i1]));
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
	bft[i0][i1]+=bf[i0][i1];
  m=0; videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) {
    n=(int)(f*bf[i0][i1]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i1%KK);
    }
  videodot(ll+ii,m++,3);
  k++;
  }  /* end for-ii */
  en=((double)(k));
//  m=0; videodot(ll+nn,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) {
    n=(int)(0.1*(bft[i0][i1]/en));
//    if (n>0) for (l=0; l<n; l++) videodot(ll+nn,m++,i0%KK);
    }
//  videodot(ll+nn,m++,3);
// for (i=0; i<TS; i++) bft[i]/=en*em;
// videocursor(0,BROW+7,0);
// for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
printf("bye from gnlsum\n");
}

/* option 'n'						*/
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

/* option 'n' */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

void neisum(ll,mm,nn) int ll, mm, nn; {
int ii, i, k, m, n, l, b[TS];
double em, en, bf[TS], bft[TS];

printf("hello from neisum\n");
asctobin();
em=((double)(mm));
for (i=0; i<TS; i++) bft[i]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  onesum(mm,TS,b);
  for (i=0; i<TS; i++) bf[i]=((double)(b[i]));
  for (i=0; i<TS; i++) bft[i]+=bf[i];
  m=0; videodot(ll+ii,m++,5);
  for (i=0; i<TS; i++) {
    n=(int)(bf[i]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i%2);
    }
  videodot(ll+ii,m++,5);
  k++;
 // if (kbdst()) {kbdin(); break;}
  }  /* end for-ii */
  en=((double)(k));
  m=0; videodot(ll+nn,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bft[i]/en);
    if (n>0) for (l=0; l<n; l++) videodot(ll+nn,m++,i%2+1);
    }
  videodot(ll+nn,m++,3);
for (i=0; i<TS; i++) bft[i]/=en*em;
//videocursor(0,BROW+7,0);
for (i=0; i<TS; i++) printf("%d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* option 'N' */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

void nbrsum(ll,mm,nn) int ll, mm, nn; {
int ii, jj, i, k, m, n, l, b[TS];
double em, en, kk, bf[TS], bft[TS];

asctobin();
em=((double)(mm));
en=((double)(nn));
for (i=0; i<TS; i++) bft[i]=0.0;
k=0;
for (jj=0; jj<nn; jj++) {
  for (i=0; i<TS; i++) bf[i]=0.0;
  for (ii=0; ii<nn; ii++) {
    onegen(mm);
    onesum(mm,TS,b);
    for (i=0; i<TS; i++) bf[i]+=((double)(b[i]));
    }  /* end for-ii */
  m=0; videodot(ll+jj,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bf[i]/en);
    if (n>0) for (l=0; l<n; l++) videodot(ll+jj,m++,i%2+1);
    }
  videodot(ll+jj,m++,3);
  for (i=0; i<TS; i++) bft[i]+=bf[i];
  k++;
  if (kbdst()) {kbdin(); break;}
  } /* end for-jj */
kk=((double)(k));
videodot(ll+jj,m++,3);
for (i=0; i<TS; i++) bft[i]/=kk*en*em;
videocursor(0,BROW+7,0);
for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* gather totalistic neighborhood sums for one generation */
/* mm = number of cells */
/* b[cc]  = sum counts  */

void onesum(mm,cc,b) int mm, cc, *b; {int i, j, k;
  printf("hello from onesum()\n");
  for (i=0; i<cc; i++) b[i]=0;
  k=arr1[mm-1]+arr1[0]; b[k]++;
  for (i=1; i<mm; i++) {k=0; for (j=i-1; j<i+1; j++) k+=arr1[j]; b[k]++;}
  printf("bye from onesum()\n");
}


/* gather general neighborhood sums for one generation */
/* mm = number of cells         */
/* b[KK][KK]  = sum counts  */
void gnrsum(mm,b) int mm, b[KK][KK]; {int i0, i1, i;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
	b[i0][i1]=0;
  for (i=0; i<mm; i++) printf("arr1[%d]=%d\n",i,arr1[i]);
  b[arr1[mm-1]][arr1[0]]++;
  for (i=1; i<mm; i++) b[arr1[i-1]][arr1[i]]++;
}

/* make a grid for the fixpoint graph  */
/* n     = dimension of grid           */
ggrid(n) int n; {int i, j;
for (i=0; i<=n; i+=10) for (j=0; j<=n; j+=10) bdot(i,j,3);}

/* put a diagonal in the fixpoint graph */
gdiag(n) int n; {int i; for (i=0; 2*i<n; i++) bdot(2*i,2*i,3);}

/* plot self-consistent probability graph */
bpanel()  {ggrid(100); gdiag(100); bgraf(100);}

/* plot self-consistent graph in Langton's parameter */
lpanel()  {ggrid(100); gdiag(100); lgraf(100);}

/* graph Bernstein polynomial */
bgraf(n) int n; {
int    i, j;
double bern(), el, em, en, p, q, t[KK];
en=(double)(n); em=1.0/en; el=1.0/((double)(KK-1));
for (p=0.0, i=0; i<n; p+=em, i++) {
  q=el*(1.0-p); for (j=1; j<KK; j++) t[j]=q;
  t[0]=p; pdot(i,(int)(en*bern(0,t)),1);
  for (j=1; j<KK; j++) t[j-1]=q; t[j]=p; bdot(i,(int)(en*bern(j,t)),1+j%3);
  }
}

/* graph Langton's parameter */
lgraf(n) int n; {int i, j;
double bern(), el, em, en, p, q, t[KK];
en=(double)(n); em=1.0/en; el=1.0/((double)(KK-1));
for (p=0.0, i=0; i<n; p+=em, i++) {
  t[0]=p; q=el*(1.0-p); for (j=1; j<KK; j++) t[j]=q;
  pdot(n-i,n-(int)(en*bern(0,t)),1);
  }
}

/* evaluate the Bernstein polynomial for component i at point t[KK] */
double bern(i,t) int i; double t[KK]; {int i0, i1; double a, b;
asctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++)
      if (binrule[i0][i1]==i) b+=t[i1];
  a+=b*t[i0];}
return a;
}

/* graph the probability differential for state l over a triangle */
pdiff(i0,n,l) int i0, n, l; {
int    i1, i2;
double p[KK], en, bern(), sqsu();
if (n==0) return;
if (i0>n) return;
en=1.0/((double)(n));
p[0]=((double)(i0))*en;
for (i1=0, p[1]=0.0; i0+i1<=n; i1++, p[1]+=en) {
for (i2=0, p[2]=0.0; i0+i1+i2<=n; i2++, p[2]+=en) {
  p[3]=1.0-p[0]-p[1]-p[2];
  switch(l) {
    case 0: videomassdot(p,psign(bern(0,p)-p[0])); break;
    case 1: videomassdot(p,psign(bern(1,p)-p[1])); break;
    case 2: videomassdot(p,psign(bern(2,p)-p[2])); break;
    case 3: videomassdot(p,psign(bern(3,p)-p[3])); break;
    case 4: videomassdot(p,lico(bern(0,p),i1+i2)); break;
    case 5: videomassdot(p,lico(bern(1,p),i2)); break;
    case 6: videomassdot(p,lico(bern(2,p),i1)); break;
    case 7: videomassdot(p,lico(bern(3,p),i1)); break;
    case 8: videomassdot(p,sqco(sqsu(p),i1+i2)); break;
    default: break;
    }
  if (kbdst()) {kbdin(); return;}
  }}
}

/* graph the probability differential for state l over a tetrahedron	*/
/* n = number of points per edge					*/
/* l = color of dot							*/

hpdiff(n,l) int n, l; {
int    i, j, k;
double bern(), x[KK], nn;

if (n==0) return;
nn=1.0/((double)(n));
for (i=0, x[0]=0.0; i<=n; i++, x[0]+=nn) {
for (j=0, x[1]=0.0; i+j<=n; j++, x[1]+=nn) {
for (k=0, x[2]=0.0; i+j+k<=n; k++, x[2]+=nn) {
  x[3]=1.0-x[0]-x[1]-x[2];
  switch(l) {
    case 0: videomassdot(x,psign(bern(0,x)-x[0])); break;
    case 1: videomassdot(x,psign(bern(1,x)-x[1])); break;
    case 2: videomassdot(x,psign(bern(2,x)-x[2])); break;
    case 3: videomassdot(x,psign(bern(3,x)-x[3])); break;
    case 4: videomassdot(x,lico(bern(0,x),i+j)); break;
    case 5: videomassdot(x,lico(bern(1,x),j)); break;
    case 6: videomassdot(x,lico(bern(2,x),i)); break;
    case 7: videomassdot(x,lico(bern(3,x),i)); break;
    case 8: videomassdot(x,sqco(sqsu(x),i+j)); break;
    default: break;
    }
  if (kbdst()) {kbdin(); return;}
  }}}
}

/* choose a color for the contour level based on sign: red, yellow, green */
psign(x) double x; {if (x>0.01) return 1; if (x<-0.01) return 2; return 3;}

/* evaluate composite Bernstein polynomial for component i at point t[KK] */
double dbern(i,t) int i; double t[KK]; {
int    i0, i1, i2;
int    j0, j1;
double a, b, c;
asctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++) {
    for (i2=0, c=0.0; i2<KK; i2++) {
	  j0=binrule[i0][i1];
	  j1=binrule[i1][i2];
          if (binrule[j0][j1]==i) c+=t[i2];
	  }
    b+=c*t[i1];}
  a+=b*t[i0];}
return a;
}

/* evaluate second composite Bernstein multinomial */
/* i for component at point t[KK] 		   */
double tbern(i,t) int i; double t[KK]; {
int    i0, i1, i2, i3;
int    j0, j1, j2;
int    k0, k1;
double a, b, c, d;
asctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++) {
    for (i2=0, c=0.0; i2<KK; i2++) {
      for (i3=0, d=0.0; i3<KK; i3++) {
	      j0=binrule[i0][i1];
	      j1=binrule[i1][i2];
	      j2=binrule[i2][i3];
	      k0=binrule[j0][j1];
	      k1=binrule[j1][j2];
              if (binrule[k0][k1]==i) d+=t[i3];
	      }
      c+=d*t[i2];}
    b+=c*t[i1];}
  a+=b*t[i0];}
return a;
}

/* evaluate third composite Bernstein multinomial */
/* i for component at point t[KK] 		   */
double fbern(i,t) int i; double t[KK]; {
int    i0, i1, i2, i3, i4;
int    j0, j1, j2, j3;
int    k0, k1, k2;
int    l0, l1;
double a, b, c, d, e;
asctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++) {
    for (i2=0, c=0.0; i2<KK; i2++) {
      for (i3=0, d=0.0; i3<KK; i3++) {
        for (i4=0, e=0.0; i4<KK; i4++) {
	      j0=binrule[i0][i1];
	      j1=binrule[i1][i2];
	      j2=binrule[i2][i3];
	      j3=binrule[i3][i4];
	      k0=binrule[j0][j1];
	      k1=binrule[j1][j2];
	      k2=binrule[j2][j3];
	      l0=binrule[k0][k1];
	      l1=binrule[k1][k2];
              if (binrule[l0][l1]==i) e+=t[i4];
	      }
        d+=e*t[i3];}
      c+=d*t[i2];}
    b+=c*t[i1];}
  a+=b*t[i0];}
return a;
}

/* evaluate fourth composite Bernstein multinomial */
/* i for component at point t[KK] 		   */
double vbern(i,t) int i; double t[KK]; {
int    i0, i1, i2, i3, i4, i5;
int    j0, j1, j2, j3, j4;
int    k0, k1, k2, k3;
int    l0, l1, l2;
int    m0, m1;
double a, b, c, d, e, f;
asctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++) {
    for (i2=0, c=0.0; i2<KK; i2++) {
      for (i3=0, d=0.0; i3<KK; i3++) {
        for (i4=0, e=0.0; i4<KK; i4++) {
          for (i5=0, f=0.0; i5<KK; i5++) {
	      j0=binrule[i0][i1];
	      j1=binrule[i1][i2];
	      j2=binrule[i2][i3];
	      j3=binrule[i3][i4];
	      j4=binrule[i4][i5];
	      k0=binrule[j0][j1];
	      k1=binrule[j1][j2];
	      k2=binrule[j2][j3];
	      k3=binrule[j3][j4];
	      l0=binrule[k0][k1];
	      l1=binrule[k1][k2];
	      l2=binrule[k2][k3];
	      m0=binrule[l0][l1];
	      m1=binrule[l1][l2];
              if (binrule[m0][m1]==i) f+=t[i5];
	      }
          e+=f*t[i4];}
        d+=e*t[i3];}
      c+=d*t[i2];}
    b+=c*t[i1];}
  a+=b*t[i0];}
return a;
}

/* calculate the squaresum of the four polynomials at a point */
double sqsu(p) double p[KK]; {int i; double s, t, bern();
for (i=0, s=0.0; i<KK; i++) {t=bern(i,p)-p[i]; s+=t*t;}
return s;
}

/* calculate the squaresum of the three polynomials at a point  */
/* for composite evolution					*/

double dsqsu(t) double t[KK]; {double u, v, w, x, dbern();
u=dbern(0,t)-t[0];
v=dbern(1,t)-t[1];
w=dbern(2,t)-t[2];
x=dbern(3,t)-t[3];
return (u*u+v*v+w*w+x*x);
}

/* calculate the squaresum of the three polynomials at a point  */
/* for composite evolution					*/

double tsqsu(t) double t[KK]; {double u, v, w, x, tbern();
u=tbern(0,t)-t[0];
v=tbern(1,t)-t[1];
w=tbern(2,t)-t[2];
x=tbern(3,t)-t[3];
return (u*u+v*v+w*w+x*x);
}

/* calculate the squaresum of the three polynomials at a point  */
/* for composite evolution					*/

double fsqsu(t) double t[KK]; {double u, v, w, x, fbern();
u=fbern(0,t)-t[0];
v=fbern(1,t)-t[1];
w=fbern(2,t)-t[2];
x=fbern(3,t)-t[3];
return (u*u+v*v+w*w+x*x);
}

/* calculate the squaresum of the three polynomials at a point  */
/* for composite evolution					*/

double vsqsu(t) double t[KK]; {double u, v, w, x, vbern();
u=vbern(0,t)-t[0];
v=vbern(1,t)-t[1];
w=vbern(2,t)-t[2];
x=vbern(3,t)-t[3];
return (u*u+v*v+w*w+x*x);
}

/* generate squaresum compatible contour values */
int sqco(t,i) int i; double t; {int l;
if (t<0.0075) l=0; else {
if (t<0.0750) {if (i%2) l=0; else l=3;} else {
if (t<0.1250) l=3; else {
if (t<0.2500) {if (i%2) l=3; else l=2;} else {
if (t<0.5000) l=2; else {
if (t<0.7500) {if (i%2) l=2; else l=1;} else {
if (t<1.0000) l=1; else {
	      if (i%2) l=1; else l=0;
   }}}}}}}
return l;
}

/* generate linear contour values */
int lico (t,i) int i; double t; {int l;
if (t<0.1) l=0; else {
if (t<0.2) {if (i%2) l=0; else l=3;} else {
if (t<0.3) l=3; else {
if (t<0.4) {if (i%2) l=3; else l=2;} else {
if (t<0.5) l=2; else {
if (t<0.6) {if (i%2) l=2; else l=1;} else {
if (t<0.7) l=1; else {
	      if (i%2) l=1; else l=0;
   }}}}}}}
return l;
}

/* graph the probability differential for state l over a triangle */
/* from iterating the mean field curve f times.			  */

pidiff(n,l,f) int n, l, f; {
int    i, j, k, l, m, sqco();
double a[KK], b[KK], c[KK], en, t, bern(), sqsu();
double u, v, w, x;
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++)
for (j=0; i+j<=n; j++)
for (k=0; i+j+k<=n; k++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=((double)(k))*en;
  a[3]=1.0-a[0]-a[1]-a[2];
  for (m=0; m<KK; m++) b[m]=a[m];
  for (m=0; m<f; m++) {
    for (l=0; l<KK; l++) c[l]=bern(l,b);
    for (l=0; l<KK; l++) b[l]=c[l];
    }
u=b[0]-a[0];
v=b[1]-a[1];
w=b[2]-a[2];
x=b[3]-a[3];
t= (u*u+v*v+w*w+x*x);
  videotrdot(a,sqco(t,i));
//  if (kbdst()) {kbdin(); return;}
  }
}

/* graph the probability differential for state l over a triangle */

pddiff(n,l) int n, l; {
int    i, j, k, sqco();
double a[KK], en, t, dsqsu();
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++)
for (j=0; i+j<=n; j++)
for (k=0; i+j+k<=n; k++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=((double)(k))*en;
  a[3]=1.0-a[0]-a[1];
  t=dsqsu(a);
// printf("%6.3f, %6.3f, %6.3f, - %6.3f\n",a[0],a[1],a[2],a[3],t); 
  videotrdot(a,sqco(t,i));
//  if (kbdst()) {kbdin(); return;}
  }
}

/* graph the probability differential for state l over a triangle */

ptdiff(n,l) int n, l; {
int    i, j, k, sqco();
double a[KK], en, t, tsqsu();
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++)
for (j=0; i+j<=n; j++)
for (k=0; i+j+k<=n; k++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=((double)(k))*en;
  a[3]=1.0-a[0]-a[1];
  t=tsqsu(a);
// printf("%6.3f, %6.3f, %6.3f, - %6.3f\n",a[0],a[1],a[2],t); 
  videotrdot(a,sqco(t,i));
//  if (kbdst()) {kbdin(); return;}
  }
}

/* graph the probability differential for state l over a triangle */

pfdiff(n,l) int n, l; {
int    i, j, k, sqco();
double a[KK], en, t, fsqsu();
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++)
for (j=0; i+j<=n; j++)
for (k=0; i+j+k<=n; k++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=((double)(k))*en;
  a[3]=1.0-a[0]-a[1];
  t=fsqsu(a);
// printf("%6.3f, %6.3f, %6.3f, - %6.3f\n",a[0],a[1],a[2],t); 
  videotrdot(a,sqco(t,i));
//  if (kbdst()) {kbdin(); return;}
  }
}

/* graph the probability differential for state l over a triangle */

pvdiff(n,l) int n, l; {
int    i, j, k, sqco();
double a[KK], en, t, vsqsu();
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++)
for (j=0; i+j<=n; j++)
for (k=0; i+j+k<=n; k++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=((double)(k))*en;
  a[3]=1.0-a[0]-a[1];
  t=vsqsu(a);
// printf("%6.3f, %6.3f, %6.3f, - %6.3f\n",a[0],a[1],a[2],t); 
  videotrdot(a,sqco(t,i));
//  if (kbdst()) {kbdin(); return;}
  }
}

/* present the frequencies of ascrule in color l */
/* percentages evolving into each value		 */
/* percentages shifting - adds to more than 100% */
/* entropy and Langton's lambda = % nonquiescent */

asfreq(l) int l; {
int    i, j, i0, i1;
int    stat[KK], stal, star;		/* statistic counts */
double staf[KK], t, pp;

pp=1.0/((double)(KK*KK));
stal=0; star=0;
for (i=0; i<KK; i++) stat[i]=0;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  j=ascrule[i0][i1]-'0';
  stat[j]++;
  if(j==i0) star++;
  if(j==i1) stal++;
  }}
videocursor(0,BROW,0);
for (i=0; i<KK; i++) staf[i]=((double)stat[i])*pp;
for (i=0; i<KK; i++) printf("%d freq - %5.2f\n",i,staf[i]);
printf("\n");
printf("left   - %5.2f\n",((double)stal)*pp);
printf("right  - %5.2f\n",((double)star)*pp);
videomassdot(staf,l);
for (i=0, t=0.0; i<KK; i++) t+=staf[i]*log(staf[i]);
videocursor(0,BROW,20);
printf("lambda:   %5.3f",1.0-staf[0]);
videocursor(0,BROW+1,20);
printf("entropy:  %5.3f",-t/log((double)KK));
videocursor(0,BROW,0);
}

/* evaluate the one-block probabilities after one generation */

onebl(x,a) double x[KK], a[KK]; {int i0, i1, j0;

for (j0=0; j0<KK; j0++) x[j0]=0.0;

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  j0=ascrule[i0][i1]-'0';
  x[j0]+=a[i0]*a[i1];
  }}
}

/* iterate the 1-block parameters to find self-consistent values */
/* graph the iterative steps in bar-chart form			 */
/* ll - initial line						 */
/* mm - length of line						 */
/* nn - number of lines						 */

oneblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i, l, m, n;
double op[KK], np[KK];
double d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i=0; i<KK; i++) {op[i]=s; for (l=0; l<n; l++) videodot(ll,m++,i);}
videodot(ll,m++,3);
videomassdot(op,2);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  onebl(np,op);
  videodot(ll+ii,m++,3);
  for (i=0; i<KK; i++) {
    n=(int)(f*np[i]); if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i);
    d=op[i]-np[i];
    e+=d*d;
    op[i]=np[i];
    }
  videodot(ll+ii,m++,3);
  if (e<=0.0000001) break;
  if (op[0]+op[1]+op[2]+op[3]<0.98) break;
  if (kbdst()) {kbdin(); break;}
  videomassdot(op,1);
  }
videocursor(0,BROW+8,0);
printf("(1-blk)"); 
for (i=0; i<KK; i++) printf("%2d:%5.3f",i,op[i]);
videomassdot(op,3);
}

/* evaluate the two-block probabilities after one generation */

twobl(x,a) double x[KK][KK], a[KK][KK]; {
int    i0, i1, i2;
int    j0, j1;
double w, b[KK];

for (j0=0; j0<KK; j0++) {for (j1=0; j1<KK; j1++) x[j0][j1]=0.0;}

for (j0=0; j0<KK; j0++) {b[j0]=0.0; for (j1=0; j1<KK; j1++) b[j0]+=a[j0][j1];}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
  j0=ascrule[i0][i1]-'0';
  j1=ascrule[i1][i2]-'0';
  w=a[i0][i1]*a[i1][i2];
  if (w!=0.0) w/=b[i1];
  x[j0][j1]+=w;
  }}}
}

/* iterate the 2-block parameters to find self-consistent values */
/* graph the iterative steps in bar-chart form			 */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

twoblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i, j, l, m, n;
double op[KK][KK], np[KK][KK];
double b[KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i=0; i<KK; i++) {
for (j=0; j<KK; j++) {
  op[i][j]=s;
  for (l=0; l<n; l++) videodot(ll,m++,j);
  }}
videodot(ll,m++,3);
hvec(b); videomassdot(b,2);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  twobl(np,op);
  videodot(ll+ii,m++,3);
  for (i=0; i<KK; i++) {
  for (j=0; j<KK; j++) {
    n=(int)(f*np[i][j]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,j);
    d=op[i][j]-np[i][j];
    e+=d<0.0?-d:d;
    op[i][j]=np[i][j];
    }}
  videodot(ll+ii,m++,3);
for (i=0; i<KK; i++) {b[i]=0.0; for (j=0; j<KK; j++) b[i]+=op[i][j];}
videomassdot(b,1);
  if (e<=0.0001) break;
  if (kbdst()) {kbdin(); break;}
  }
videocursor(0,BROW+8,0);
printf("(2-blk)"); 
for (i=0; i<KK; i++) printf("%2d:%5.3f",i,b[i]);
videomassdot(b,3);
}

/* evaluate the three-block probabilities after one generation */

thrbl(x,a) double x[KK][KK][KK], a[KK][KK][KK]; {
int    i0, i1, i2, i3;
int    j0, j1, j2;
double w, b[KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
  for (j2=0; j2<KK; j2++) x[j0][j1][j2]=0.0;
  b[j0][j1]=0.0;
  for (j2=0; j2<KK; j2++) b[j0][j1]+=a[j0][j1][j2];  
  }}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
  j0=ascrule[i0][i1]-'0';
  j1=ascrule[i1][i2]-'0';
  j2=ascrule[i2][i3]-'0';
  w=a[i0][i1][i2]*a[i1][i2][i3];
  if (w!=0.0) w/=b[i1][i2];
  x[j0][j1][j2]+=w;
  }}}}
}

/* iterate the 3-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

thrblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i, j, k, l, m, n;
double op[KK][KK][KK], np[KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i=0; i<KK; i++) {
for (j=0; j<KK; j++) {
for (k=0; k<KK; k++) {
  op[i][j][k]=s;
  for (l=0; l<n; l++) videodot(ll,m++,k);
  }}}
videodot(ll,m++,3);
hvec(b); videomassdot(b,2);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  thrbl(np,op);
  videodot(ll+ii,m++,3);
  for (i=0; i<KK; i++) {
  for (j=0; j<KK; j++) {
  for (k=0; k<KK; k++) {
    n=(int)(f*np[i][j][k]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,k);
    d=op[i][j][k]-np[i][j][k];
    e+=d<0.0?-d:d;
    op[i][j][k]=np[i][j][k];
    }}}
  videodot(ll+ii,m++,3);

  for (i=0; i<KK; i++) {
    b[i]=0.0;
    for (j=0; j<KK; j++) {
    for (k=0; k<KK; k++) {
      b[i]+=op[i][j][k];
      }} }
  videomassdot(b,1);
  if (e<=0.0001) break;
  if (kbdst()) {kbdin(); break;}
  }

videocursor(0,BROW+8,0);
printf("(3-blk)"); 
for (i=0; i<KK; i++) printf("%2d:%5.3f",i,b[i]);
videomassdot(b,3);

for (i=0; i<KK; i++) {
for (j=0; j<KK; j++) {
  bb[i][j]=0.0;
  for (k=0; k<KK; k++) {
    bb[i][j]+=op[i][j][k];
    }} }

/*videocursor(0,BROW+9,0);*/
/*for (i=0; i<KK; i++) for (j=0; j<KK; j++)*/ 
/*  printf("%1d%1d:%5.3f ",i,j,bb[i][j]);*/
/*videodot(199-(int)(100.0*bb[1][1]),BORG+(int)(100.0*b[1]),2);*/
}

/* evaluate the four-block probabilities after one generation */

foubl(x,a) double x[KK][KK][KK][KK], a[KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4;
int    j0, j1, j2, j3;
double w, b[KK][KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
for (j2=0; j2<KK; j2++) {
  for (j3=0; j3<KK; j3++) x[j0][j1][j2][j3]=0.0;
  b[j0][j1][j2]=0.0;
  for (j3=0; j3<KK; j3++) b[j0][j1][j2]+=a[j0][j1][j2][j3];  
  }}}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
  j0=ascrule[i0][i1]-'0';
  j1=ascrule[i1][i2]-'0';
  j2=ascrule[i2][i3]-'0';
  j3=ascrule[i3][i4]-'0';
  w=a[i0][i1][i2][i3]*a[i1][i2][i3][i4];
  if (w!=0.0) w/=b[i1][i2][i3];
  x[j0][j1][j2][j3]+=w;
  }}}}}
}

/* iterate the 4-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

foublfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i0, i1, i2, i3, l, m, n;
double op[KK][KK][KK][KK], np[KK][KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
  op[i0][i1][i2][i3]=s;
  for (l=0; l<n; l++) videodot(ll,m++,i3);
  }}}}
videodot(ll,m++,3);
hvec(b); videomassdot(b,2);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  foubl(np,op);
  videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) {
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
    n=(int)(f*np[i0][i1][i2][i3]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i3);
    d=op[i0][i1][i2][i3]-np[i0][i1][i2][i3];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3]=np[i0][i1][i2][i3];
    }}}}
  videodot(ll+ii,m++,3);

  for (i0=0; i0<KK; i0++) {
    b[i0]=0.0;
    for (i1=0; i1<KK; i1++) {
    for (i2=0; i2<KK; i2++) {
    for (i3=0; i3<KK; i3++) {
      b[i0]+=op[i0][i1][i2][i3];
      }}} }
  videomassdot(b,1);
  if (e<=0.0001) break;
  if (kbdst()) {kbdin(); break;}
  }

videocursor(0,BROW+8,0);
printf("(4-blk)"); 
for (i0=0; i0<KK; i0++) printf("%2d:%5.3f",i0,b[i0]);
videomassdot(b,3);

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
    bb[i0][i1]+=op[i0][i1][i2][i3];
    }} }}

/*videocursor(0,BROW+9,0);*/
/*for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++)*/ 
/*  printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);*/
/*videodot(199-(int)(100.0*bb[1][1]),BORG+(int)(100.0*b[1]),2);*/
}

/* evaluate the five-block probabilities after one generation */

fivbl(x,a) double x[KK][KK][KK][KK][KK], a[KK][KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5;
int    j0, j1, j2, j3, j4;
double w, b[KK][KK][KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
for (j2=0; j2<KK; j2++) {
for (j3=0; j3<KK; j3++) {
  for (j4=0; j4<KK; j4++) x[j0][j1][j2][j3][j4]=0.0;
  b[j0][j1][j2][j3]=0.0;
  for (j4=0; j4<KK; j4++) b[j0][j1][j2][j3]+=a[j0][j1][j2][j3][j4];  
  }}}}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
for (i5=0; i5<KK; i5++) {
  j0=ascrule[i0][i1]-'0';
  j1=ascrule[i1][i2]-'0';
  j2=ascrule[i2][i3]-'0';
  j3=ascrule[i3][i4]-'0';
  j4=ascrule[i4][i5]-'0';
  w=a[i0][i1][i2][i3][i4]*a[i1][i2][i3][i4][i5];
  if (w!=0.0) w/=b[i1][i2][i3][i4];
  x[j0][j1][j2][j3][j4]+=w;
  }}}}}}
}

/* iterate the 5-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

fivblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i0, i1, i2, i3, i4, l, m, n;
double op[KK][KK][KK][KK][KK], np[KK][KK][KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
  op[i0][i1][i2][i3][i4]=s;
  for (l=0; l<n; l++) videodot(ll,m++,i4);
  }}}}}
videodot(ll,m++,3);
hvec(b); videomassdot(b,2);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  fivbl(np,op);
  videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) {
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
    n=(int)(f*np[i0][i1][i2][i3][i4]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i4);
    d=op[i0][i1][i2][i3][i4]-np[i0][i1][i2][i3][i4];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3][i4]=np[i0][i1][i2][i3][i4];
    }}}}}
  videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) {
    b[i0]=0.0;
    for (i1=0; i1<KK; i1++) {
    for (i2=0; i2<KK; i2++) {
    for (i3=0; i3<KK; i3++) {
    for (i4=0; i4<KK; i4++) {
    b[i0]+=op[i0][i1][i2][i3][i4];
    }}}} }
  videomassdot(b,1);
  if (e<=0.0001) break;
  if (kbdst()) {kbdin(); break;}
  }
videocursor(0,BROW+8,0);
printf("(5-blk)"); 
for (i0=0; i0<KK; i0++) printf("%2d:%5.3f",i0,b[i0]);
videomassdot(b,3);

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
    bb[i0][i1]+=op[i0][i1][i2][i3][i4];
    }} }}}

/*videocursor(0,BROW+9,0);*/
/*for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) */
/*  printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);*/
/*videodot(199-(int)(100.0*bb[1][1]),BORG+(int)(100.0*b[1]),2);*/
}

/* end nxprob4h.c */
