
/* nxrijn6h.c						*/
/* program for LCA3h option 'd'				*/
/* display de Bruijn diagram - 1st, 2nd, & 3rd gen	*/
/* Harold V. McIntosh, 20 February 1988			*/
/* 21 February 1988 - add j0=i0; to Pass 4's		*/
/* 26 September 1989 - show evolution of periodic conf	*/
/* 2 October 1989 - cycl3h.c incorporated into rijn3h.c	*/
/* 20 November 1989 - correct loop in kwait		*/
/* 22 June 1994 - adapt to NeXT				*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1994		*/
/*    Harold V. McIntosh		*/

# define JX  0.50  /* x-center, de Bruijn diagram	*/
# define JY  0.50  /* y-center, de Bruijn diagram	*/
# define RI 0.425  /* inner radius, de Bruijn diagram	*/
# define RO 0.475  /* outer radius, de Bruijn diagram	*/

# include <math.h>
# include "nxlcau6h.h"
# include "nxrijn6h.h"
# import  <dpsclient/psops.h>
# import  <appkit/graphics.h>
# import  <appkit/appkit.h>

extern NXColor colst[KK];

extern char arule[DS], arr1[AL], arr2[AL], arr3[AL];
extern char ascrule[KK][KK];		/* ASCII transition values */
extern int  binrule[KK][KK];
extern int  ix0, iy0;			/* origin for pen moves */
extern int  dbno;			/* node in de Bruijn dgm */

char                        haarry[KK][KK];
char                       aarry[KK][KK][KK];
char                    hbarry[KK][KK][KK][KK];
char                   barry[KK][KK][KK][KK][KK];
char                hcarry[KK][KK][KK][KK][KK][KK];
char               carry[KK][KK][KK][KK][KK][KK][KK];


int  dd;		/* # points in de Bruijn ring	*/
int  mc, nc, nl;	/* column & line control	*/

double  cntr[XDB][2];

/* edit the de Bruijn diagram		*/
/* g = number of generation (0==2nd)	*/
/* s = shift distance or constant val	*/
void edijn(g,s,p,r) int g, s, p, r; {

   switch(g+'a') {

    case 'A': xxpass1(s); break;
    case 'B': xpass1(s); break;
    case 'C': yypass1(s); break;
    case 'D': ypass1(s); break;
    case 'E': zzpass1(s); break;
    case 'F': zpass1(s); break;

    case 'a': switch (s) {
      case 0: ccpass1(8,p,r);  break;
      case 1: bbpass1(4,p,r);  break;
      case 2: aapass1(0,p,r);  break;
      case 3: aapass1(1,p,r);  break;
      case 4: bbpass1(5,p,r);  break;
      case 5: ccpass1(9,p,r); break;
    default: break;} break;

    case 'b': switch (s) {
      case 0: cpass1(9,p,r); break;
      case 1: bpass1(5,p,r); break;
      case 2: apass1(0,p,r); break;
      case 3: apass1(1,p,r); break;
      case 4: apass1(2,p,r); break;
      case 5: bpass1(6,p,r); break;
      case 6: cpass1(10,p,r); break;
    default: break;} break;

    case 'c': switch (s) {
      case 0: ccpass1(6,p,r); break;
      case 1: bbpass1(0,p,r); break;
      case 2: bbpass1(1,p,r); break;
      case 3: bbpass1(2,p,r); break;
      case 4: bbpass1(3,p,r); break;
      case 5: ccpass1(7,p,r); break;
    default: break;} break;

    case 'd': switch (s) {
      case 0: cpass1(7,p,r); break;
      case 1: bpass1(0,p,r); break;
      case 2: bpass1(1,p,r); break;
      case 3: bpass1(2,p,r); break;
      case 4: bpass1(3,p,r); break;
      case 5: bpass1(4,p,r); break;
      case 6: cpass1(8,p,r); break;
    default: break;} break;

    case 'e': switch (s) {
      case 0: ccpass1(0,p,r); break;
      case 1: ccpass1(1,p,r); break;
      case 2: ccpass1(2,p,r); break;
      case 3: ccpass1(3,p,r); break;
      case 4: ccpass1(4,p,r); break;
      case 5: ccpass1(5,p,r); break;
    default: break;} break;

  default: break;
  }
}


/* edit the Pass 6 constant displays */
void edshc(g,c) int g, c; {
  switch(g) {
    case 0: aapass5(c); break;
    case 1: apass5(c); break;
    case 2: bbpass5(c); break;
    case 3: bpass5(c); break;
    case 4: ccpass5(c); break;
    case 5: cpass5(c); break;
    default: break;}
}

/* edit the Pass 4 and Pass 5 shift displays */
void edshf(g,s,c) int g, s, c; {
    switch(shflev(g,s)) {
    case 0: aapass5(c); break;
    case 1: apass5(c); break;
    case 2: bbpass5(c); break;
    case 3: bpass5(c); break;
    case 4: ccpass5(c); break;
    case 5: cpass5(c); break;
    default: break;
    } /* end switch */
}

/* find the level of de Bruijn diagram appropriate to g, s  */
/* g = row, so g=0 means 2nd generation			    */
/* s = column, so 2s-5 is center for g even, s-3 for g odd  */
/* returns order of underlying de Bruijn diagram, 0=lowest  */
int shflev(g,s) int g, s; {int  i, k;
  k=g%2;
  if (k==0) {
    i=2*s-5;
    i=i<0?-i:i;
    i=(i+1)/2;
    if (i<=(1+g/2)) i=g; else i=2*(i-1);
    }
  else {
    i=s-3;
    i=i<0?-i:i;
    if (i<=(g+1)/2) i=g; else i=2*i-1;
    }
  return i;
}

/* edit the Pass 4 and Pass 5 shift displays */
void edshg(g,s) int g, s; {
    switch(shflev(g,s)) {
    case 0: aapass6(); break;
    case 1: apass6(); break;
    case 2: bbpass6(); break;
    case 3: bpass6(); break;
    case 4: ccpass6(); break;
    case 5: cpass6(); break;
    default: break;
    } /* end switch */
}

/* edit the Pass 4 and Pass 5 constant displays */
void edcst(s) int s; {
    switch(s) {
    case 0: aapass6(); break;
    case 1: apass6(); break;
    case 2: bbpass6(); break;
    case 3: bpass6(); break;
    case 4: ccpass6(); break;
    case 5: cpass6(); break;
    default: break;
    } /* end switch */
}

/* increment ix */ 
void incix(ix,n) int *ix, n; {int i; 
  for (i=0; i<n; i++) {
    ix[i]++;
    if (ix[i]>=KK) ix[i]=0; else return;
    }
}

/* selected node in ix */ 
void nodix(ix,n) int *ix, n; {int i, j; 
  j=dbno;
  for (i=0; i<n; i++) {ix[n-i-1]=j%KK; j/=KK;}
}

/* random number in ix */ 
void ranix(ix,n) int *ix, n; {int i, j; 
  j=random();
  for (i=0; i<n; i++) {ix[i]=j%KK; j/=KK;}
}

/* clear ix */ 
void zerix(ix,n) int *ix, n; {int i; for (i=0; i<n; i++) ix[i]=0;}

/* find index of nearest center to x, y (de Bruijn) */
int nearest(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<dd; i++) {
  ex=x-cntr[i][0];
  wy=y-cntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* generate a de Bruijn diagram with m nodes, n links per node, color l */
void ijn(m,n,l) int m, n, l; {
int    i, j;
double ii, jj, nn, x, y, t, h;
nn=(double)(n);
t=6.28318/((double)(m));
h=0.5*t;
for (i=0; i<m; i++) {
  ii=(double)(i);
  for (j=0; j<n; j++) {
    jj=(double)(j);
    x=JX-RO*sin(ii*t+h);
    y=JY-RO*cos(ii*t+h);
    videoline(x,y,0);
    x=JX-RO*sin((nn*ii+jj)*t+h);
    y=JY-RO*cos((nn*ii+jj)*t+h);
    videoline(x,y,l);
    }
  }
}

/* insert a link into a de Bruijn diagram */
/* u - initial vertex     */
/* v - terminal vertex    */
/* l - color of link	  */

void lijnk(u,v,l) int u, v, l; {
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, w, t, sx, sy;

	if (l<KK) NXSetColor(colst[l]); else PSsetgray(NX_BLACK);
	r=10.3;
	w=3.33;
	ex=cntr[u][0];
	wy=cntr[u][1];
	x=cntr[v][0];
	y=cntr[v][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	sx=-w*sin(t);
	sy= w*cos(t);
	eks=x-r*cos(t)+sx;
	wye=y-r*sin(t)+sy;
	PSmoveto(ex+sx,wy+sy);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSmoveto(x+10.3,y);
	PSarc(x, y, 10.3, 0.0, 360.0);
	PSstroke();

   PSsetgray(NX_BLACK); 

   ex=cntr[u][0];
   wy=cntr[u][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",u);
   PSshow(s);

   ex=cntr[v][0];
   wy=cntr[v][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",v);
   PSshow(s);
   PSstroke();

}

/* generate a de Bruijn diagram showing ancestors of state k in color l */
void sijn(k,l) int k, l; {
int    i0, i1;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
  if (k==ascrule[i0][i1]-'0') lijnk(i0,i1,l);
}

/* generate a de Bruijn diagram showing the (1,k) links in color l */
void zijn(m,k,l) int m, k, l; {
int    i, i0, i1;
double th, x, y, t, h;
t=6.28318/((double)(m));
h=0.5*t;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  i=ascrule[i0][i1]-'0';
  if ((k==0 && i==i0) || (k==1 && i==i1)) {
    th=((double)(i0))*t+h;
    x=JX-RI*sin(th);
    y=JY-RI*cos(th);
    videoline(x,y,0);
    th=((double)(i1))*t+h;
    x=JX-RO*sin(th);
    y=JY-RO*cos(th);
    videoline(x,y,l);
    }	/* end if    */
  }}   /* end for's */
}	/* end zijn  */

/* plot graph on video screen 		   */
/* move pen from present position to (x,y) */
/* raised if l=0; color l if l>0	   */
void videoline(x,y,l) double x, y; int l; {
if (l==0) PSmoveto(400.0*x,400.0*y);
else PSlineto(400.0*x,400.0*y);
}

/* - - - - - - - - - d e   B r u i j n - - - - - - - - - - - - - - - - - - */

/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

/* ONE6h.C */

/* Pass 1a analyzes all the configurations which fulfill (-1+l,1) */
void aapass1(l,p,r) int l, p, r; {
int  ix[RR], m;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++) {
  m=permut(ix[l],p);
  haarry[ix[0]][ix[1]]=binrule[ix[0]][ix[1]]==m?'Y':'N';
  }
if (r) {aapass2i(); aapass2o();}
}

/* Pass 1x analyzes all the configurations mapping into a constant */
void xxpass1(l) int l; {
int  ix[RR];
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++) {
  haarry[ix[0]][ix[1]]=binrule[ix[0]][ix[1]]==l?'Y':'N';
  }
aapass2i();
aapass2o();
}

/* Pass 2i flags all links with an incoming arrow */
/* Pass 2o flags all links with an outgoing arrow */
/* Then pass 3 discards all unflagged links */
/* Passes 2 and 3 alternate until no change is observed */

void aapass2i() {int i0, i1, m;
do {
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  if ((haarry[i0][i1]&0x5F)=='Y') for (m=0; m<KK; m++) haarry[i1][m]|=0x20;
  } while (aapass3()!=0);
}

void aapass2o() {int i0, i1, m;
do {
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
if ((haarry[i0][i1]&0x5F)=='Y') {for (m=0; m<KK; m++) haarry[m][i0]|=0x20;}
}} } while (aapass3()!=0); }

/* Pass 3 - erase flags, mark survivors, count changes */

int aapass3() {int i0, i1, l;
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
  switch (haarry[i0][i1]) {
    case 'y': haarry[i0][i1]='Y'; break;
    case 'Y': haarry[i0][i1]='N'; l=1; break;
    case 'n': case 'N': haarry[i0][i1]='N'; break;
    default: break; }
return l;
}

/* Pass4 - print loops which remain */
void aapass4() {
int i0, i1;
int j0, j1;
int k, l, m;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
j0=i0; j1=i1; l=0; m=0;
do {
        if (haarry[0][j1]=='Y')
    {haarry[0][j1]='y';
    k=j1; j1=0; m=1;}
  else {if (haarry[1][j1]=='Y')
    {haarry[1][j1]='y';
    k=j1; j1=1; m=1;}
  else {if (haarry[2][j1]=='Y')
    {haarry[2][j1]='y';
    k=j1; j1=2; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=k;} }}}
  } while (l==0);
l=0; 
m=0;
do {
        if (haarry[j0][0]=='y')
   {aprf(j0,0);
   haarry[j0][0]='N';
   j0=0; m=1;}
  else {if (haarry[j0][1]=='y')
   {aprf(j0,1);
   haarry[j0][1]='N';
   j0=1; m=1;}
  else {if (haarry[j0][2]=='y')
   {aprf(j0,2);
   haarry[j0][2]='N';
   j0=2; m=1;}
  else {l=1;}}}
  } while (l==0);
l=0;
do {
        if (haarry[j0][0]=='Y')
   {aprf(j0,0);
   haarry[j0][0]='N';
   j0=0; m=1;}
  else {if (haarry[j0][1]=='Y')
   {aprf(j0,1);
   haarry[j0][1]='N';
   j0=1; m=1;}
  else {if (haarry[j0][2]=='Y')
   {aprf(j0,2);
   haarry[j0][2]='N';
   j0=2; m=1;}
  else {l=1;}}}
  } while (l==0);
}}
}

/* Pass 5 - show evolution of sample loops */
void aapass5(c) char c; {
int i0, i1;
int j1;
int hl;
int b[KK], irry[RR], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR); break;
    case 'r': ranix(irry,RR); break;
    case 'n': nodix(irry,RR-1);
	i0=irry[0];
	for (i=j=0; j<KK; j++) if (haarry[i0][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[1]=b[l];
	break;
    case 'i': incix(irry,RR); break;
    default:  zerix(irry,RR); break;}
  i0=irry[0]; i1=irry[1];
for (k=0; k<9; k++) {
  if (haarry[i0][i1]=='Y') break;
  incix(irry,RR);
  i0=irry[0]; i1=irry[1];
  }
hl=AL/2;
arr1[hl-1]=i0;
arr1[hl]=i1;
j1=i1;
for (k=hl-2; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (haarry[j][i0]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i0; i0=arr1[k]=b[l];
  }
i1=j1;
for (k=hl+1; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (haarry[i1][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i0; i0=arr1[k]=b[l];
  }
}

void aapass6() {int i0, i1;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++) {
  if(haarry[i0][i1]=='Y') lijnk(i0,i1,i1);
  }
}

/* ONE21.C */

/* Pass 1a analyzes all the configurations which fulfil (-1+l,1) */
void apass1(l,p,r) int l, p, r; {
int  ix[RR+1], jx[RR], m;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++) {
  jx[0]=binrule[ix[0]][ix[1]];
  jx[1]=binrule[ix[1]][ix[2]];
  m=permut(ix[l],p);
  aarry[ix[0]][ix[1]][ix[2]]=binrule[jx[0]][jx[1]]==m?'Y':'N';
  }
if (r) {apass2i(); apass2o();}
}

/* Pass 1x analyzes all the configurations mapping into a constant */
void xpass1(l) int l; {
int  ix[RR+1], jx[RR], m;
m=l;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++) {
  jx[0]=binrule[ix[0]][ix[1]];
  jx[1]=binrule[ix[1]][ix[2]];
  aarry[ix[0]][ix[1]][ix[2]]=binrule[jx[0]][jx[1]]==m?'Y':'N';
  }
apass2i();
apass2o();
}

/* Pass 2i flags all links with an incoming arrow */
/* Pass 2o flags all links with an outgoing arrow */
/* Then pass 3 discards all unflagged links */
/* Passes 2 and 3 alternate until no change is observed */

void apass2i() {int i,j,k,m;
do {
for (i=0; i<KK; i++)
for (j=0; j<KK; j++)
for (k=0; k<KK; k++) {
if ((aarry[i][j][k]&0x5F)=='Y') {for (m=0; m<KK; m++) aarry[j][k][m]|=0x20;}
}
 } while (apass3()!=0); }

void apass2o() {int i,j,k,m;
do {
for (i=0; i<KK; i++)
for (j=0; j<KK; j++)
for (k=0; k<KK; k++) {
if ((aarry[i][j][k]&0x5F)=='Y') {for (m=0; m<KK; m++) aarry[m][i][j]|=0x20;}
}
 } while (apass3()!=0); }

/* Pass 3 - erase flags, mark survivors, count changes */

int apass3() {int i,j,k,l;
l=0;
for (i=0; i<KK; i++)
for (j=0; j<KK; j++)
for (k=0; k<KK; k++) {
  switch (aarry[i][j][k]) {
    case 'y': aarry[i][j][k]='Y'; break;
    case 'Y': aarry[i][j][k]='N'; l=1; break;
    case 'n': case 'N': aarry[i][j][k]='N'; break;
    default: break; }
}
return l;
}

/* Pass4 - print loops which remain */
void apass4() {
int i0, i1, i2;
int j0, j1, j2;
int k, l, m;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
j0=i0; j1=i1; j2=i2; l=0; m=0;
do {
        if (aarry[0][j1][j2]=='Y')
    {aarry[0][j1][j2]='y';
    k=j2; j2=j1; j1=0; m=1;}
  else {if (aarry[1][j1][j2]=='Y')
    {aarry[1][j1][j2]='y';
    k=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=k;} }}
  } while (l==0);
l=0; 
m=0;
do {
        if (aarry[j0][j1][0]=='y')
   {aprf(j0,j1,0);
   aarry[j0][j1][0]='N';
   j0=j1; j1=0; m=1;}
  else {if (aarry[j0][j1][1]=='y')
   {aprf(j0,j1,1);
   aarry[j0][j1][1]='N';
   j0=j1; j1=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (aarry[j0][j1][0]=='Y')
   {aprf(j0,j1,0);
   aarry[j0][j1][0]='N';
   j0=j1; j1=0; m=1;}
  else {if (aarry[j0][j1][1]=='Y')
   {aprf(j0,j1,1);
   aarry[j0][j1][1]='N';
   j0=j1; j1=1; m=1;}
  else {l=1; }}
  } while (l==0);
}}}
}

/* print one link of the de Bruijn diagram */
void aprf(i,j,k) int i, j, k; {
// kwait(1);
// printf("%1d",i);
// printf("%1d",j);
// printf("-");
// printf("%1d",k);
// printf("  ");
lijnk(i*KK+j,j*KK+k,3);
}

/* Pass 5 - show evolution of sample loops */
void apass5(c) char c; {
int i0, i1, i2;
int j1;
int hl;
int b[KK], irry[RR+1], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR+1); break;
    case 'r': ranix(irry,RR+1); break;
    case 'n': nodix(irry,RR);
	i0=irry[0]; i1=irry[1];
	for (i=j=0; j<KK; j++) if (aarry[i0][i1][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[2]=b[l];
	break;
    case 'i': incix(irry,RR+1); break;
    default:  zerix(irry,RR+1); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2];
for (k=0; k<9; k++) {
  if (aarry[i0][i1][i2]=='Y') break;
  incix(irry,RR+1);
  i0=irry[0]; i1=irry[1]; i2=irry[2];
  }
hl=AL/2;
arr1[hl-1]=i0;
arr1[hl]=i1;
arr1[hl+1]=i2;
j1=i1;
for (k=hl-2; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (aarry[j][i0][i1]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i0; i0=arr1[k]=b[l];
  }
i1=j1;
for (k=hl+2; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (aarry[i1][i2][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=arr1[k]=b[l];
  }
}

void apass6() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++) {
  if(aarry[i0][i1][i2]=='Y') lijnk(i0*KK+i1,i1*KK+i2,i2);
  }
}

/* TWO2H.C */

/* mark sequences conforming to (-2+l,2) */
void bbpass1(l,p,r) int l, p, r; {
int ix[RR+2], jx[RR+1], kx[RR], lx;
int i, m, n;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++) {
  for (i=0; i<RR+1; i++) jx[i]=binrule[ix[i]][ix[i+1]];
  for (i=0; i<RR; i++)   kx[i]=binrule[jx[i]][jx[i+1]];
			 lx   =binrule[kx[0]][kx[1]];
  if (l<4)  {m=ix[l]; n=lx;}
  if (l==4) {m=ix[0]; n=jx[1];}
  if (l==5) {m=ix[3]; n=jx[1];}
  hbarry[ix[0]][ix[1]][ix[2]][ix[3]]=m==permut(n,p)?'Y':'N';
  }
if (r) {bbpass2i(); bbpass2o();}
}

/* Pass 1y analyzes all the configurations mapping into a constant */
void yypass1(l) int l; {
int ix[RR+2], jx[RR+1], kx[RR], lx;
int i;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++) {
  for (i=0; i<RR+1; i++) jx[i]=binrule[ix[i]][ix[i+1]];
  for (i=0; i<RR; i++)   kx[i]=binrule[jx[i]][jx[i+1]];
			 lx=binrule[kx[0]][kx[1]];
  hbarry[ix[0]][ix[1]][ix[2]][ix[3]]=lx==l?'Y':'N';
  }
bbpass2i();
bbpass2o();
}

/* Passs 2i flags links which have an incoming arrow */
void bbpass2i() {
int i0, i1, i2, i3, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  if ((hbarry[i0][i1][i2][i3]&0x5F)=='Y')
    {for (m=0; m<KK; m++) hbarry[i1][i2][i3][m]|=0x20;}
  }
} while (bbpass3()!=0); }

/* Passs 2o flags links which have an outgoing arrow */
void bbpass2o() {
int i0, i1, i2, i3, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  if ((hbarry[i0][i1][i2][i3]&0x5F)=='Y')
    {for (m=0; m<KK; m++) hbarry[m][i0][i1][i2]|=0x20;}
  }
} while (bbpass3()!=0); }

/*  Pass 3 erases flags, marks survivors, counts changes */
int bbpass3() {
int i0, i1, i2, i3, l;
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  switch (hbarry[i0][i1][i2][i3]) {
    case 'y': hbarry[i0][i1][i2][i3]='Y'; break;
    case 'Y': hbarry[i0][i1][i2][i3]='N'; l=1; break;
    case 'n': case 'N': hbarry[i0][i1][i2][i3]='N'; break;
    default: break; }
  }
return l;
}

/* Pass 4 prints loops which remain */
void bbpass4() {
int i0, i1, i2, i3;
int j0, j1, j2, j3, k, l, m;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++){
j0=i0; j1=i1; j2=i2; j3=i3; l=0; m=0;
do {
        if (hbarry[0][j1][j2][j3]=='Y')
    {hbarry[0][j1][j2][j3]='y';
    k=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (hbarry[1][j1][j2][j3]=='Y')
    {hbarry[1][j1][j2][j3]='y';
    k=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=j3; j3=k;} }}
  } while (l==0);
l=0; 
m=0;
do {
        if (hbarry[j0][j1][j2][0]=='y')
   {bprf(j0,j1,j2,0);
   hbarry[j0][j1][j2][0]='N';
   j0=j1; j1=j2; j2=j3; m=1;}
  else {if (hbarry[j0][j1][j2][1]=='y')
   {bprf(j0,j1,j2,1);
   hbarry[j0][j1][j2][1]='N';
   j0=j1; j1=j2; j2=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (hbarry[j0][j1][j2][0]=='Y')
   {bprf(j0,j1,j2,0);
   hbarry[j0][j1][j2][0]='N';
   j0=j1; j1=j2; j2=0; m=1;}
  else {if (hbarry[j0][j1][j2][1]=='Y')
   {bprf(j0,j1,j2,1);
   hbarry[j0][j1][j2][1]='N';
   j0=j1; j1=j2; j2=1; m=1;}
  else {l=1; }}
  } while (l==0);
}
}

/* print one of the links in a chain */
void bbprf(i0,i1,i2,i3,i4) int i0, i1, i2, i3, i4; {
// kwait(1);
// printf("%1d",i0);
// printf("%1d",i1);
// printf("%1d",i2);
// printf("%1d",i3);
// printf("-");
// printf("%1d",i4);
// printf(" ");
lijnk(((i0*KK+i1)*KK+i2)*KK+i3,((i1*KK+i2)*KK+i3)*KK+i4,3);
}

/* Pass 5 - show evolution of sample loops */
void bbpass5(c) char c; {
int i0, i1, i2, i3;
int j1, j2, j3;
int hl;
int b[KK], irry[RR+2], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR+2); break;
    case 'n': nodix(irry,RR+1);
	i0=irry[0]; i1=irry[1]; i2=irry[2];
	for (i=j=0; j<KK; j++) if (hbarry[i0][i1][i2][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[3]=b[l];
	break;
    case 'i': incix(irry,RR+2); break;
    default:  zerix(irry,RR+2); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3];;
for (k=0; k<81; k++) {
  if (hbarry[i0][i1][i2][i3]=='Y') break;
  incix(irry,RR+2);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3];;
  }
hl=AL/2;
arr1[hl-2]=i0;
arr1[hl-1]=i1;
arr1[hl]=i2;
arr1[hl+1]=i3;
j1=i1; j2=i2; j3=i3;
for (k=hl-3; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (hbarry[j][i0][i1][i2]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
        k=k;
  }
i1=j1; i2=j2; i3=j3;
for (k=hl+2; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (hbarry[i1][i2][i3][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=i3; i3=arr1[k]=b[l];
         i=i;
  }
}

void bbpass6() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++){
  if(hbarry[i0][i1][i2][i3]=='Y')
    lijnk((i0*KK+i1)*KK+i2,(i1*KK+i2)*KK+i3,i3);
  }
}

/* TWO6h.C */

/* mark sequences conforming to (-2+l,2) */
void bpass1(l,p,r) int l, p, r; {
int ix[RR+3], jx[RR+2], kx[RR+1], lx[RR], mx;
int i, m, n;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++) {
  for (i=0; i<RR+2; i++) jx[i]=binrule[ix[i]][ix[i+1]];
  for (i=0; i<RR+1; i++) kx[i]=binrule[jx[i]][jx[i+1]];
  for (i=0; i<  RR; i++) lx[i]=binrule[kx[i]][kx[i+1]];
			 mx   =binrule[lx[0]][lx[1]];
  if (l<5)  {m=ix[l]; n=mx;}
  if (l==5) {m=ix[0]; n=jx[2];}
  if (l==6) {m=ix[4]; n=jx[2];}
  barry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]]=m==permut(n,p)?'Y':'N';
  }
if (r) {bpass2i(); bpass2o();}
}

/* Pass 1y analyzes all the configurations mapping into a constant */
void ypass1(l) int l; {
int ix[RR+3], jx[RR+2], kx[RR+1], lx[RR], mx;
int i;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++) {
  for (i=0; i<RR+2; i++) jx[i]=binrule[ix[i]][ix[i+1]];
  for (i=0; i<RR+1; i++) kx[i]=binrule[jx[i]][jx[i+1]];
  for (i=0; i<RR  ; i++) lx[i]=binrule[kx[i]][kx[i+1]];
			 mx   =binrule[lx[0]][lx[1]];
  barry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]]=mx==l?'Y':'N';
  }
bpass2i();
bpass2o();
}

/* Passs 2i flags links which have an incoming arrow */
void bpass2i() {
int i0, i1, i2, i3, i4, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
if ((barry[i0][i1][i2][i3][i4]&0x5F)=='Y')
 {for (m=0; m<KK; m++) barry[i1][i2][i3][i4][m]|=0x20;}
}
} while (bpass3()!=0); }

/* Passs 2o flags links which have an outgoing arrow */
void bpass2o() {
int i0, i1, i2, i3, i4, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
if ((barry[i0][i1][i2][i3][i4]&0x5F)=='Y')
 {for (m=0; m<KK; m++) barry[m][i0][i1][i2][i3]|=0x20;}
}
} while (bpass3()!=0); }

/*  Pass 3 erases flags, marks survivors, counts changes */
int bpass3() {
int i0, i1, i2, i3, i4, l;
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
switch (barry[i0][i1][i2][i3][i4]) {
    case 'y': barry[i0][i1][i2][i3][i4]='Y'; break;
    case 'Y': barry[i0][i1][i2][i3][i4]='N'; l=1; break;
    case 'n': case 'N': barry[i0][i1][i2][i3][i4]='N'; break;
    default: break; }
}
return l;
}

/* Pass 4 prints loops which remain */
void bpass4() {
int i0, i1, i2, i3, i4;
int j0, j1, j2, j3, j4, k, l, m;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4; l=0; m=0;
do {
        if (barry[0][j1][j2][j3][j4]=='Y')
    {barry[0][j1][j2][j3][j4]='y';
    k=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (barry[1][j1][j2][j3][j4]=='Y')
    {barry[1][j1][j2][j3][j4]='y';
    k=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=j3; j3=j4; j4=k;} }}
  } while (l==0);
l=0; 
m=0;
do {
        if (barry[j0][j1][j2][j3][0]=='y')
   {bprf(j0,j1,j2,j3,0);
   barry[j0][j1][j2][j3][0]='N';
   j0=j1; j1=j2; j2=j3; j3=0; m=1;}
  else {if (barry[j0][j1][j2][j3][1]=='y')
   {bprf(j0,j1,j2,j3,1);
   barry[j0][j1][j2][j3][1]='N';
   j0=j1; j1=j2; j2=j3; j3=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (barry[j0][j1][j2][j3][0]=='Y')
   {bprf(j0,j1,j2,j3,0);
   barry[j0][j1][j2][j3][0]='N';
   j0=j1; j1=j2; j2=j3; j3=0; m=1;}
  else {if (barry[j0][j1][j2][j3][1]=='Y')
   {bprf(j0,j1,j2,j3,1);
   barry[j0][j1][j2][j3][1]='N';
   j0=j1; j1=j2; j2=j3; j3=1; m=1;}
  else {l=1; }}
  } while (l==0);
}
}

/* print one of the links in a chain */
void bprf(i0,i1,i2,i3,i4) int i0, i1, i2, i3, i4; {
// kwait(1);
// printf("%1d",i0);
// printf("%1d",i1);
// printf("%1d",i2);
// printf("%1d",i3);
// printf("-");
// printf("%1d",i4);
// printf(" ");
lijnk(((i0*KK+i1)*KK+i2)*KK+i3,((i1*KK+i2)*KK+i3)*KK+i4,3);
}

/* Pass 5 - show evolution of sample loops */
void bpass5(c) char c; {
int i0, i1, i2, i3, i4;
int j1, j2, j3;
int hl;
int b[KK], irry[RR+3], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR+3); break;
    case 'r': ranix(irry,RR+3); break;
    case 'n': nodix(irry,RR+2);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3];
  for (i=j=0; j<KK; j++) if (barry[i0][i1][i2][i3][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[4]=b[l];
	break;
    case 'i': incix(irry,RR+3); break;
    default:  zerix(irry,RR+3); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4];
for (k=0; k<81; k++) {
  if (barry[i0][i1][i2][i3][i4]=='Y') break;
  incix(irry,RR+3);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4];
  }
hl=AL/2;
arr1[hl-2]=i0;
arr1[hl-1]=i1;
arr1[hl]=i2;
arr1[hl+1]=i3;
arr1[hl+2]=i4;
j1=i1; j2=i2; j3=i3;
for (k=hl-3; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (barry[j][i0][i1][i2][i3]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
        k=k;
  }
i1=j1; i2=j2; i3=j3;
for (k=hl+3; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (barry[i1][i2][i3][i4][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=i3; i3=i4; i4=arr1[k]=b[l];
         i=i;
  }
}

void bpass6() {int i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  if(barry[i0][i1][i2][i3][i4]=='Y')
    lijnk(((i0*KK+i1)*KK+i2)*KK+i3,
             ((i1*KK+i2)*KK+i3)*KK+i4,i4);
  }
}

/* THREE6h.C */

/* mark sequences conforming to (-3+l,3) */
void ccpass1(l,p,r) int l, p, r; {
int  ix[RR+4], jx[RR+3], kx[RR+2], lx[RR+1], mx[RR], nx;
int  i, m, n;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++) {
  for (i=0; i<RR+3; i++) jx[i]=binrule[ix[i]][ix[i+1]];
  for (i=0; i<RR+2; i++) kx[i]=binrule[jx[i]][jx[i+1]];
  for (i=0; i<RR+1; i++) lx[i]=binrule[kx[i]][kx[i+1]];
  for (i=0; i<RR  ; i++) mx[i]=binrule[lx[i]][lx[i+1]];
			 nx   =binrule[mx[0]][mx[1]];
  if (l<=5) {m=ix[l]; n=nx;}
  if (l==6) {m=ix[1]; n=jx[2];}
  if (l==7) {m=ix[4]; n=jx[2];}
  if (l==8) {m=ix[0]; n=kx[1];}
  if (l==9) {m=ix[5]; n=kx[1];}
hcarry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]]=m==permut(n,p)?'Y':'N';
}
if (r) {ccpass2i(); ccpass2o();}
}

/* Pass 1z analyzes all the configurations mapping into a constant */
void zzpass1(l) int l; {
int  ix[RR+4], jx[RR+3], kx[RR+2], lx[RR+1], mx[RR], nx;
int  i;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++) {
  for (i=0; i<RR+3; i++) jx[i]=binrule[ix[i]][ix[i+1]];
  for (i=0; i<RR+2; i++) kx[i]=binrule[jx[i]][jx[i+1]];
  for (i=0; i<RR+1; i++) lx[i]=binrule[kx[i]][kx[i+1]];
  for (i=0; i<RR  ; i++) mx[i]=binrule[lx[i]][lx[i+1]];
			 nx   =binrule[mx[0]][mx[1]];
hcarry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]]=nx==l?'Y':'N';
}
ccpass2i();
ccpass2o();
}

/* Pass 2i flags links which have an incoming arrow */
void ccpass2i() {
int i0, i1, i2, i3, i4, i5, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++) {
if ((hcarry[i0][i1][i2][i3][i4][i5]&0x5F)=='Y')
 {for (m=0; m<KK; m++) hcarry[i1][i2][i3][i4][i5][m]|=0x20;}
}
} while (ccpass3()!=0); }

/* Pass 2o flags links which have an outgoing arrow */
void ccpass2o() {
int i0, i1, i2, i3, i4, i5, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++) {
if ((hcarry[i0][i1][i2][i3][i4][i5]&0x5F)=='Y')
 {for (m=0; m<KK; m++) hcarry[m][i0][i1][i2][i3][i4]|=0x20;}
}
} while (ccpass3()!=0); }

/*  Pass 3 erases flags, marks survivors, counts channges */
int ccpass3() {
int i0, i1, i2, i3, i4, i5, l;
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++) {
switch (hcarry[i0][i1][i2][i3][i4][i5]) {
    case 'y': hcarry[i0][i1][i2][i3][i4][i5]='Y'; break;
    case 'Y': hcarry[i0][i1][i2][i3][i4][i5]='N'; l=1; break;
    case 'n': case 'N': hcarry[i0][i1][i2][i3][i4][i5]='N'; break;
    default: break; }
}
return l; }

/* Pass 4 prints loops which remain */
void ccpass4() {
int i0, i1, i2, i3, i4, i5;
int j0, j1, j2, j3, j4, j5;
int  k, l, m;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++){
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4; j5=i5;
l=0;
m=0;
do {
        if (hcarry[0][j1][j2][j3][j4][j5]=='Y')
    {hcarry[0][j1][j2][j3][j4][j5]='y';
    k=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (hcarry[1][j1][j2][j3][j4][j5]=='Y')
    {hcarry[1][j1][j2][j3][j4][j5]='y';
    k=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1;
    if (m==1) {j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=k;}
    }}
  } while (l==0);
l=0; 
m=0;
do {
        if (hcarry[j0][j1][j2][j3][j4][0]=='y')
   {ccprf(j0,j1,j2,j3,j4,0);
   hcarry[j0][j1][j2][j3][j4][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4==0; m=1;}
  else {if (hcarry[j0][j1][j2][j3][j4][1]=='y')
   {ccprf(j0,j1,j2,j3,j4,1);
   hcarry[j0][j1][j2][j3][j4][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (hcarry[j0][j1][j2][j3][j4][0]=='Y')
   {ccprf(j0,j1,j2,j3,j4,0);
   hcarry[j0][j1][j2][j3][j4][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=0; m=1;}
  else {if (hcarry[j0][j1][j2][j3][j4][1]=='Y')
   {ccprf(j0,j1,j2,j3,j4,1);
   hcarry[j0][j1][j2][j3][j4][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=1; m=1;}
  else {l=1;}}
  } while (l==0);
}
}

/* print one of the individual links in a chain */
void ccprf(i0,i1,i2,i3,i4,i5)
int i0, i1, i2, i3, i4, i5; {int i, j;
// kwait(1);
// printf("%1d",i0);
// printf("%1d",i1);
// printf("%1d",i2);
// printf("%1d",i3);
// printf("%1d",i4);
// printf("%1d",i5);
// printf("-");
// printf("%1d",i6);
// printf(" ");
i=((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4);
j=((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5);
lijnk(i,j,3);
}

/* Pass 5 - show evolution of sample loops */
void ccpass5(c) char c; {
int i0, i1, i2, i3, i4, i5;
int j1, j2, j3, j4, j5;
int hl;
int b[KK], irry[RR+4], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR+4); break;
    case 'r': ranix(irry,RR+4); break;
    case 'n': nodix(irry,RR+3);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4];
  for (i=j=0; j<KK; j++) if (hcarry[i0][i1][i2][i3][i4][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[5]=b[l];
	break;
    case 'i': incix(irry,RR+4); break;
    default:  zerix(irry,RR+4); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2];
  i3=irry[3]; i4=irry[4]; i5=irry[5];
for (k=0; k<729; k++) {
  if (hcarry[i0][i1][i2][i3][i4][i5]=='Y') break;
  incix(irry,RR+4);
  i0=irry[0]; i1=irry[1]; i2=irry[2];
  i3=irry[3]; i4=irry[4]; i5=irry[5];
  }
hl=AL/2;
arr1[hl-3]=i0;
arr1[hl-2]=i1;
arr1[hl-1]=i2;
arr1[hl]=i3;
arr1[hl+1]=i4;
arr1[hl+2]=i5;
j1=i1; j2=i2; j3=i3; j4=i4; j5=i5;
for (k=hl-4; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (hcarry[j][i0][i1][i2][i3][i4]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i5=i4; i4=i3; i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
  }
i1=j1; i2=j2; i3=j3; i4=j4; i5=j5;
for (k=hl+3; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (hcarry[i1][i2][i3][i4][i5][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=i3; i3=i4; i4=i5; i5=arr1[k]=b[l];
  }
}

void ccpass6() {int i, j, i0, i1, i2, i3, i4, i5;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++) {
  i=((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4);
  j=((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5);
  if(hcarry[i0][i1][i2][i3][i4][i5]=='Y') lijnk(i,j,i5);
  }
}

/* THREE21.C */

/* mark sequences conforming to (-3+l,3) */
void cpass1(l,p,r) int l, p, r; {
int  ix[RR+5], jx[RR+4], kx[RR+3], lx[RR+2], mx[RR+1], nx[RR], ox;
int  i, m, n;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++)
for (ix[6]=0; ix[6]<KK; ix[6]++) {
  for (i=0; i<RR+4; i++) jx[i]=binrule[ix[i]][ix[i+1]];
  for (i=0; i<RR+3; i++) kx[i]=binrule[jx[i]][jx[i+1]];
  for (i=0; i<RR+2; i++) lx[i]=binrule[kx[i]][kx[i+1]];
  for (i=0; i<RR+1; i++) mx[i]=binrule[lx[i]][lx[i+1]];
  for (i=0; i<RR  ; i++) nx[i]=binrule[mx[i]][mx[i+1]];
			 ox   =binrule[nx[0]][nx[1]];
  if (l<=6)  {m=ix[l]; n=ox;}
  if (l==7)  {m=ix[1]; n=jx[2];}
  if (l==8)  {m=ix[5]; n=jx[2];}
  if (l==9)  {m=ix[0]; n=kx[1];}
  if (l==10) {m=ix[6]; n=kx[1];}
carry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]=m==permut(n,p)?'Y':'N';
}
if (r) {cpass2i(); cpass2o();}
}

/* Pass 1z analyzes all the configurations mapping into a constant */
void zpass1(l) int l; {
int  ix[RR+5], jx[RR+4], kx[RR+3], lx[RR+2], mx[RR+1], nx[RR], ox;
int  i;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++)
for (ix[6]=0; ix[6]<KK; ix[6]++) {
  for (i=0; i<RR+4; i++) jx[i]=binrule[ix[i]][ix[i+1]];
  for (i=0; i<RR+3; i++) kx[i]=binrule[jx[i]][jx[i+1]];
  for (i=0; i<RR+2; i++) lx[i]=binrule[kx[i]][kx[i+1]];
  for (i=0; i<RR+1; i++) mx[i]=binrule[lx[i]][lx[i+1]];
  for (i=0; i<RR  ; i++) nx[i]=binrule[mx[i]][mx[i+1]];
			 ox   =binrule[nx[0]][nx[1]];
carry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]=ox==l?'Y':'N';
}
cpass2i();
cpass2o();
}

/* Pass 2i flags links which have an incoming arrow */
void cpass2i() {
int i0, i1, i2, i3, i4, i5, i6, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
if ((carry[i0][i1][i2][i3][i4][i5][i6]&0x5F)=='Y')
 {for (m=0; m<KK; m++) carry[i1][i2][i3][i4][i5][i6][m]|=0x20;}
}
} while (cpass3()!=0); }

/* Pass 2o flags links which have an outgoing arrow */
void cpass2o() {
int i0, i1, i2, i3, i4, i5, i6, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
if ((carry[i0][i1][i2][i3][i4][i5][i6]&0x5F)=='Y')
 {for (m=0; m<KK; m++) carry[m][i0][i1][i2][i3][i4][i5]|=0x20;}
}
} while (cpass3()!=0); }

/*  Pass 3 erases flags, marks survivors, counts channges */
int cpass3() {
int i0, i1, i2, i3, i4, i5, i6, l;
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
switch (carry[i0][i1][i2][i3][i4][i5][i6]) {
    case 'y': carry[i0][i1][i2][i3][i4][i5][i6]='Y'; break;
    case 'Y': carry[i0][i1][i2][i3][i4][i5][i6]='N'; l=1; break;
    case 'n': case 'N': carry[i0][i1][i2][i3][i4][i5][i6]='N'; break;
    default: break; }
}
return l; }

/* Pass 4 prints loops which remain */
void cpass4() {
int i0, i1, i2, i3, i4, i5, i6;
int j0, j1, j2, j3, j4, j5, j6;
int  k, l, m;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4; j5=i5; j6=i6;
l=0;
m=0;
do {
        if (carry[0][j1][j2][j3][j4][j5][j6]=='Y')
    {carry[0][j1][j2][j3][j4][j5][j6]='y';
    k=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (carry[1][j1][j2][j3][j4][j5][j6]=='Y')
    {carry[1][j1][j2][j3][j4][j5][j6]='y';
    k=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1;
    if (m==1) {j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=k;}
    }}
  } while (l==0);
l=0; 
m=0;
do {
        if (carry[j0][j1][j2][j3][j4][j5][0]=='y')
   {cprf(j0,j1,j2,j3,j4,j5,0);
   carry[j0][j1][j2][j3][j4][j5][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=0; m=1;}
  else {if (carry[j0][j1][j2][j3][j4][j5][1]=='y')
   {cprf(j0,j1,j2,j3,j4,j5,1);
   carry[j0][j1][j2][j3][j4][j5][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (carry[j0][j1][j2][j3][j4][j5][0]=='Y')
   {cprf(j0,j1,j2,j3,j4,j5,0);
   carry[j0][j1][j2][j3][j4][j5][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=0; m=1;}
  else {if (carry[j0][j1][j2][j3][j4][j5][1]=='Y')
   {cprf(j0,j1,j2,j3,j4,j5,1);
   carry[j0][j1][j2][j3][j4][j5][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=1; m=1;}
  else {l=1;}}
  } while (l==0);
}
}

/* print one of the individual links in a chain */
void cprf(i0,i1,i2,i3,i4,i5,i6)
int i0, i1, i2, i3, i4, i5, i6; {int i, j;
//kwait(1);
// printf("%1d",i0);
// printf("%1d",i1);
// printf("%1d",i2);
// printf("%1d",i3);
// printf("%1d",i4);
// printf("%1d",i5);
// printf("-");
// printf("%1d",i6);
// printf(" ");
i=((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5;
j=((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6;
lijnk(i,j,3);
}

/* Pass 5 - show evolution of sample loops */
void cpass5(c) char c; {
int i0, i1, i2, i3, i4, i5, i6;
int j1, j2, j3, j4, j5;
int hl;
int b[KK], irry[RR+5], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR+5); break;
    case 'r': ranix(irry,RR+5); break;
    case 'n': nodix(irry,RR+4);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4]; i5=irry[5];
  for (i=j=0; j<KK; j++) 
     if (carry[i0][i1][i2][i3][i4][i5][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  irry[6]=b[l];
  break;
    case 'i': incix(irry,RR+5); break;
    default:  zerix(irry,RR+5); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2];
  i3=irry[3]; i4=irry[4]; i5=irry[5]; i6=irry[6];
for (k=0; k<729; k++) {
  if (carry[i0][i1][i2][i3][i4][i5][i6]=='Y') break;
  incix(irry,RR+5);
  i0=irry[0]; i1=irry[1]; i2=irry[2];
  i3=irry[3]; i4=irry[4]; i5=irry[5]; i6=irry[6];
  }
hl=AL/2;
arr1[hl-3]=i0;
arr1[hl-2]=i1;
arr1[hl-1]=i2;
arr1[hl]=i3;
arr1[hl+1]=i4;
arr1[hl+2]=i5;
arr1[hl+3]=i6;
j1=i1; j2=i2; j3=i3; j4=i4; j5=i5;
for (k=hl-4; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (carry[j][i0][i1][i2][i3][i4][i5]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i5=i4; i4=i3; i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
  }
i1=j1; i2=j2; i3=j3; i4=j4; i5=j5;
for (k=hl+4; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (carry[i1][i2][i3][i4][i5][i6][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=i3; i3=i4; i4=i5; i5=i6; i6=arr1[k]=b[l];
  }
}

void cpass6() {int i, j, i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  i=((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5;
  j=((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6;
  if(carry[i0][i1][i2][i3][i4][i5][i6]=='Y') lijnk(i,j,i6);
  }
}

/* end nxrijn6h.c */
