/* rec.c -- G. Cisneros, 4.91; rev. 3.10.91 */
# include <ctype.h>
# include <setjmp.h>
# include "rec.h"

/*	Values returned by rec_c and rec_x:	*/

/*	 1	True exit (or succesful comp)	*/
/*	 0	False exit			*/
/*	-1	Exit through underscore	op.	*/
/*	-2	Missing program or expression	*/
/*	-3	Premature end of source		*/
/*	-4	Compilation stack overflow	*/
/*	-5	Compilation stack underflow	*/
/*	-6	Program too big			*/
/*	-7	Unbalanced comment		*/
/*	-8	Missing program or expression	*/
/*	-9	Unbalanced left brace		*/
/*	-10	Unbalanced right bracket	*/
/*	-11	Unbalanced right brace		*/
/*	-12	Undefined subroutine (@x)	*/
/*	-101	Keyboard interrupt during exec	*/

static Inst *ilc;	/* Instruction location counter (compilation p.c.) */
Inst *r_pc;		/* Program counter (execution p.c.) */
static Inst *ilink, *tlink, *flink;	/* Iteration, True, False links */

int r_usub(), jump(), r_xpred();

static Inst qusub = r_usub;

# define NPPSTACK	1024	/* compiler pointer stack size */

static Inst *ppstack[NPPSTACK];
static Inst **ppstkp = ppstack;

static Inst *progend;

Inst *r_vst[128] = {		/* variable and subroutine table */
        NULIP, NULIP, NULIP, NULIP, NULIP, NULIP, NULIP, NULIP,
        NULIP, NULIP, NULIP, NULIP, NULIP, NULIP, NULIP, NULIP,
        NULIP, NULIP, NULIP, NULIP, NULIP, NULIP, NULIP, NULIP,
        NULIP, NULIP, NULIP, NULIP, NULIP, NULIP, NULIP, NULIP,
        NULIP, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
	&qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub,
        &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, &qusub, NULIP
};

static struct fptbl *r_dtbl;

char *r_pbufp;		/* pseudo-input buffer */
int r_pbufl;		/* pseudo-input buffer length */

int r_lastchar;

/* Optional: String parameter operator (module STR.C)  */
int r_cstrp();   /* compilation of string parameter */
int r_lstrp();   /* execution (loading) of a string paremeter */
/* ...............................................*/


static jmp_buf r_env;

rec_c(str, prog, plen, table)
char *str;		/* program source */
Inst *prog;		/* pointer to array for object code */
int plen;		/* size of object code array */
struct fptbl *table;	/* predicate table */
{
    int c;

    r_dtbl = table;
    if ((c = setjmp(r_env)))
	return -c;
    progend = prog + plen;	/* define program area upper bound*/
    r_pbufl = strlen(str);
    r_pbufp = str;
    ilink = (Inst *) 0;		/* mark outermost level */
    ilc = prog;			/* starting instruction location counter */
    ppstkp = ppstack;		/* compilation stack */
    r_compile();
  /* for (r_pc = prog; r_pc < ilc; r_pc++)
    printf("%x: %lx\n",(int)r_pc,(long) *r_pc); */
    r_pc = ilc;
    return 1;
}

static int ctc_flag;

# ifndef MSDOS
# include <signal.h>
static void ctc_handl()
{
    ctc_flag = 1;
}
# endif

# ifdef MSDOS
# include <dos.h>
static int ctc_handl()
{
    ctc_flag = 1;
    return 1;
}
# endif


rec_x(pc)
Inst *pc;
{
    int c;

# ifndef MSDOS
    void (*oldhdl)();

    ctc_flag = 0;
    oldhdl = signal(SIGINT, ctc_handl); /* intercept ctrl-C handling */
# endif

# ifdef MSDOS
    void interrupt (*oldhdl)();

    ctc_flag = 0;
    oldhdl = getvect(0x23);
    /* ctc_get();*/			/* intercept ctrl-C handling */
    ctrlbrk(ctc_handl);
# endif

    if (!(c = -setjmp(r_env)))	/* save machine state for error exit */
	c = execute(pc);

# ifndef MSDOS
    signal(SIGINT, oldhdl);             /* release ctrl-C handling */
# endif

# ifdef MSDOS
    setvect(0x23, oldhdl);
    /* ctc_rel();*/			/* release ctrl-C handling */
# endif

    return c;
}

r_initvst()
{
    int c;

    for (c = 0; c < '!'; c++)
        r_vst[c] = NULIP;
    for (c = '!'; c < '~'; c++)
	r_vst[c] = &qusub;
    r_vst[127] = NULIP;
}

r_usub()
{
    r_errterm(12);
}

r_compile()
{
    if ((r_lastchar = left()) == -1)
	r_errterm(2);
    r_ungetch();
    loop();
}

r_errterm(msgno)
int msgno;
{
    longjmp(r_env, msgno);
}

static int execute(p)
Inst *p;
{
    for (r_pc = p; *r_pc != True && *r_pc != False; )
	(*(*r_pc++))();
    return (int) *r_pc;
}

static loop()
{
    int c;

    while ((c = r_nxtchar()) != -1)
	if (isgraph(c)) {
	    r_lastchar = c;
	    c -= ' ';	/* table starts at entry for blank */
	    (*r_dtbl[c].cfun)(r_dtbl[c].xfun);
	    if (ilink == (Inst *) 0)
		return;
	}
    r_errterm(3);
}

static void pppush(iptr)
Inst *iptr;
{
    if (ppstkp >= &ppstack[NPPSTACK])
	r_errterm(4);
    *ppstkp++ = iptr;
}

static Inst *pppop()
{
    if (ppstkp <= ppstack)
	r_errterm(5);
    return *--ppstkp;
}

r_code(func)
Inst func;
{
    if (ilc >= progend)
	r_errterm(6);
    *ilc++ = func;
}

static left()
{
    int c, notwarn=1;

    while((c = r_nxtchar()) != -1)
	if (c == '(' || c == '{')
	    return c;
	else if (c == '[')
	    r_comment();
	else if (c != ' ' && c != '\t' && c != '\n' && notwarn) {
	/*  fprintf(stderr,"Nonblank character(s) before left delimiter\n"); */
	    notwarn = 0;
	}
    return c;
}

r_comment()
{
    int c;

    while ((c = r_nxtchar()) != -1 && c != ']')
	if (c == '[')
	    r_comment();
    if (c == ']')
	return;
    r_errterm(7);
}

r_noopc(dum)
Inst dum;
{
}

r_lpar(dum)
Inst dum;
{
    pppush(tlink);
    pppush(flink);
    pppush(ilink);

    ilink = ilc;
    flink = tlink = (Inst *) 0;
}

r_rpar(dum)
Inst dum;
{
    Inst *cjump();
    static Inst *svf;

    ilink = pppop();
    if (ilink == (Inst *) 0) {	/* outermost level */
	r_code(False);		/* False exit for the expression */
	fillch(&flink);		/* fill the last inner False chain */
	fillch(&tlink);		/* fill the True chain */
	r_code(True);		/* both chains point to a True exit */
	flink = pppop();	/* pop "outer" False link */
    } else {
	svf = flink;		/* save inner False link */
	flink = pppop();	/* pop outer False link */
	flink = cjump(flink);	/* link into outer False chain */
	fillch(&svf);		/* fill the last inner False chain */
	fillch(&tlink);		/* fill the True chain */
    }
    tlink = pppop();		/* pop outer True link */
}

r_colon(dum)
Inst dum;
{
    Inst *cjump();

    cjump(ilink);		/* jump to start of expression */
    fillch(&flink);		/* fill the False chain */
}

r_semicol(dum)
Inst dum;
{
    Inst *cjump();

    tlink = cjump(tlink);	/* link into True chain */
    fillch(&flink);		/* fill the False chain */
}

static Inst *cjump(addr)
Inst *addr;
{
    Inst *oilc;

    r_code(jump);
    oilc = ilc;
    r_code((Inst) addr);
    return oilc;
}

static fillch(ipt)
Inst **ipt;
{
    register Inst *ihold;
    register Inst *iptr;

    iptr = *ipt;
    while (iptr != (Inst *) 0) {
	ihold = *((Inst **) iptr);
	*iptr = (Inst) ilc;
	iptr = ihold;
    }
    *ipt = iptr;
}

r_linkf()
{
    Inst *oilc;

    oilc = ilc;
    r_code((Inst) flink);	/* link into False chain */
    flink = oilc;
}

r_pred(func)
Inst func;
{
    r_code(r_xpred);
    r_code(func);
    r_linkf();		/* link into False chain */
}

r_pred1(func)
Inst func;
{
    r_code(r_xpred);
    r_code(func);
    r_code((Inst) r_nxtchar());
    r_linkf();		/* link into False chain */
}

r_oper1(func)		/* operator followed by one ASCII arg */
Inst func;
{
    r_code(func);
    r_code((Inst) r_nxtchar());
}

r_pred2(func)		/* predicate followed by two ASCII args */
Inst func;
{
    int c;

    r_code(r_xpred);
    r_code(func);
    c = r_nxtchar() << 8;
    r_code((Inst) (c | r_nxtchar()));
    r_linkf();		/* link into False chain */
}

r_oper2(func)		/* operator followed by two ASCII args */
Inst func;
{
    int c;

    r_code(func);
    c = r_nxtchar() << 8;
    r_code((Inst) (c | r_nxtchar()));
}

r_lbrace(func)
Inst func;
{
    static int c;
    int subcount=0;
    Inst *stilc, *svilc, **tppstp;

    r_code(jump);
    stilc = ilc++;
    pppush(ilink);	/* save current iteration pointer */
    ilink = (Inst *) 0;
    tppstp = ppstkp;	/* subroutine list needed in FIFO order */
    while (1) {
	if ((r_lastchar = c = left()) == -1)
	    r_errterm(8);
	r_ungetch();
	svilc = ilc;
	loop();
	while ((c = r_nxtchar()) != -1 && !isgraph(c))
	    ;
	if (c == -1)
	    r_errterm(9);
	if (c == '}')
	    break;
	pppush((Inst *) c);
	pppush(svilc);
	subcount++;
    }	
    *stilc = (Inst) ilc;
    ppstkp = tppstp;	/* remove list from stack */
    r_code(func);	/* function performing brace execution */
    r_code((Inst) subcount);
    for (; subcount > 0; subcount--) {	/* in order of appearance */
	r_code((Inst) *tppstp++);	/* name */
	r_code((Inst) *tppstp++);	/* entry point */
    }
    r_code((Inst) svilc);	/* start of main */
    ilink = pppop();	/* recover iteration pointer */
    if (ilink == (Inst *) 0) {	/* braces are outside of parentheses */
	r_code(False);
	r_code(True);
    } else {		/* braces are inside an expression */
	svilc = ilc;
	r_code((Inst) flink);	/* link into False chain */
	flink = svilc;
    }
}

r_ubrack(dum)
Inst dum;
{
    r_errterm(10);
}

r_ubrace(dum)
Inst dum;
{
    r_errterm(11);
}

r_nxtchar()
{
    if (r_pbufl--) return *r_pbufp++;
    r_pbufl = 0;
    return -1;
}

r_ungetch()
{
    r_pbufl++;
    r_pbufp--;
}

static jump()		/* unconditional jumps */
{
# ifdef MSDOS
    bdos(11,0,0);	/* check keyboard status (ignore return value) */
# endif

    if (ctc_flag)
	    r_errterm(101);	/* keyboard interrupt with ctrl-C */
    r_pc = (Inst *) *r_pc;	/* new pc in current cell */
}

r_xpred()
{
    if ((*(*r_pc++))())
    	r_pc++;			/* True: skip over False link */
    else
	r_pc = (Inst *) *r_pc;	/* False: take False link */
}

int r_call()		/* @: subroutine call */
{
    Inst *svpc;
    register int name;

    name = (int) *r_pc++;	/* get subroutine name */
    svpc = r_pc;		/* save return address */
    r_pc = r_vst[name];
    name = execute(r_pc);
    r_pc = svpc;
    return name;
}

r_quit()		/* _: panic exit */
{
    r_errterm(1);
}

r_xbrace()		/* {: execute a brace-enclosed program */
{
    static int name;
    register int i;
    int nsub, retcond;
    Inst *svpc;

    nsub = (int) *r_pc++;
    for(i=0; i<nsub; i++) {
	name = (int) *r_pc++;
	pppush(r_vst[name]);	/* save previous definition */
	r_vst[name] = (Inst *) *r_pc++;	/* load new definition */
    }
    svpc = r_pc;			/* save for restoration */
    retcond = execute(*r_pc);
    r_pc = svpc++;		/* point svpc at False exit */
    for (i=0; i<nsub; i++) {
	--r_pc;
	name = (int) *--r_pc;
	r_vst[name] = pppop();	/* restore in reverse */
    }
    r_pc = retcond ? svpc+1 : ((*svpc == False) ? svpc : (Inst *) *svpc);
	/* return: skip if True, else jump (dereference) if not outer brace */
}
